'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import effort
import task
import category
import note


def viewerTypes():
    ''' Return the available viewer types, using the names as used in the 
        settings. '''
    return ('timelineviewer', 'squaretaskviewer', 'taskviewer', 
        'taskstatsviewer', 'noteviewer', 'categoryviewer', 'effortviewer', 
        'calendarviewer', 'hierarchicalcalendarviewer', 'effortviewerforselectedtasks')


class addViewers(object):  # pylint: disable=C0103, R0903
    ''' addViewers is a class masquerading as a method. It's a class because
        that makes it easier to split the work over different methods that
        use the same instance variables. '''
    
    floating = False  # Start viewers floating? Not when restoring layout
    
    def __init__(self, viewer_container, task_file, settings):
        self.__viewer_container = viewer_container
        self.__settings = settings
        self.__viewer_init_args = (viewer_container.containerWidget, task_file, 
                                   settings)
        self.__add_all_viewers()
        
    def __add_all_viewers(self):
        ''' Open viewers as saved previously in the settings. '''
        self.__add_viewers(task.TaskViewer)
        self.__add_viewers(task.TaskStatsViewer)
        self.__add_viewers(task.SquareTaskViewer)
        self.__add_viewers(task.TimelineViewer)
        self.__add_viewers(task.CalendarViewer)
        self.__add_viewers(task.HierarchicalCalendarViewer)
        self.__add_viewers(effort.EffortViewer)
        self.__add_viewers(effort.EffortViewerForSelectedTasks)
        self.__add_viewers(category.CategoryViewer)
        self.__add_viewers(note.NoteViewer)

    def __add_viewers(self, viewer_class):
        ''' Open viewers of the specified viewer class as saved previously in
            the settings. '''
        number_of_viewers_to_add = self._number_of_viewers_to_add(viewer_class)
        for _ in range(number_of_viewers_to_add):
            viewer_instance = viewer_class(*self.__viewer_init_args, 
                                           **self._viewer_kwargs(viewer_class))
            self.__viewer_container.addViewer(viewer_instance, 
                                              floating=self.floating)
    
    def _number_of_viewers_to_add(self, viewer_class):
        ''' Return the number of viewers of the specified viewer class the 
            user has opened previously. '''
        return self.__settings.getint('view', 
                                      viewer_class.__name__.lower() + 'count')

    def _viewer_kwargs(self, viewer_class):  # pylint: disable=R0201
        ''' Return the keyword arguments to be passed to the viewer 
            initializer. '''
        return dict(viewerContainer=self.__viewer_container) if issubclass(viewer_class, effort.EffortViewerForSelectedTasks) else dict()


class addOneViewer(addViewers):  # pylint: disable=C0103, R0903
    ''' addOneViewer is a class masquerading as a method to add one viewer
        of a specified viewer class. '''
    
    floating = True  # Start viewer floating? Yes when opening a new viewer
    
    def __init__(self, viewer_container, task_file, settings, viewer_class, 
                 **kwargs):
        self.__viewer_class = viewer_class
        self.__kwargs = kwargs
        super(addOneViewer, self).__init__(viewer_container, task_file, 
                                           settings)
        
    def _number_of_viewers_to_add(self, viewer_class):
        return 1 if viewer_class == self.__viewer_class else 0
        
    def _viewer_kwargs(self, viewer_class):
        kwargs = super(addOneViewer, self)._viewer_kwargs(viewer_class)
        kwargs.update(self.__kwargs)
        return kwargs



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test, sys, os, ConfigParser, StringIO
from taskcoachlib import config, meta
from taskcoachlib.thirdparty.pubsub import pub


class SettingsTestCase(test.TestCase):
    def setUp(self):
        self.settings = config.Settings(load=False)

    def tearDown(self):
        super(SettingsTestCase, self).tearDown()
        del self.settings


class SettingsTest(SettingsTestCase):
    def testDefaults(self):
        self.failUnless(self.settings.has_section('view'))
        self.assertEqual(True, self.settings.getboolean('view', 'statusbar'))

    def testSet(self):
        self.settings.setvalue('view', 'toolbar', (16, 16))
        self.assertEqual((16, 16), self.settings.gettuple('view', 'toolbar'))

    def testGetList_EmptyByDefault(self):
        self.assertEqual([], self.settings.getlist('file', 'recentfiles'))

    def testSetList_Empty(self):
        self.settings.setlist('file', 'recentfiles', [])
        self.assertEqual([], self.settings.getlist('file', 'recentfiles'))
    
    def testSetList_SimpleStrings(self):
        recentfiles = ['abc', 'C:\Documents And Settings\Whatever']
        self.settings.setlist('file', 'recentfiles', recentfiles)
        self.assertEqual(recentfiles, 
                         self.settings.getlist('file', 'recentfiles'))
        
    def testSetList_UnicodeStrings(self):
        recentfiles = ['√É¬ºmlaut', '√é¬£√é¬ø√é¬º√é¬∑ √è‚Ä°√èÔøΩ√é¬µ√é¬µ√é¬∫']
        self.settings.setlist('file', 'recentfiles', recentfiles)
        self.assertEqual(recentfiles, 
                         self.settings.getlist('file', 'recentfiles'))
        
    def testGetNonExistingSettingFromSection1ReturnsDefault(self):
        self.settings.add_section('effortviewer1')
        self.settings.set('effortviewer', 'columnwidths', 'dict(subject=10)')
        self.assertEqual(eval(config.defaults.defaults['effortviewer']['columnwidths']), 
            self.settings.getdict('effortviewer1', 'columnwidths'))

    def testGetNonExistingSettingFromSection2ReturnsDefault(self):
        self.settings.add_section('effortviewer1')
        self.settings.add_section('effortviewer2')
        self.settings.set('effortviewer1', 'columnwidths', 'dict(subject=10)')
        self.assertEqual(eval(config.defaults.defaults['effortviewer']['columnwidths']), 
            self.settings.getdict('effortviewer2', 'columnwidths'))
        
    def testGetNonExistingSettingFromSection2RaisesException(self):
        self.settings.add_section('effortviewer1')
        self.settings.add_section('effortviewer2')
        self.assertRaises(ConfigParser.NoOptionError,
            self.settings.get, 'effortviewer2', 'nonexisting')
        
    def testGetNonExistingSectionRaisesException(self):
        self.assertRaises(ConfigParser.NoSectionError, self.settings.get, 'bla', 'bla')

    def testAddSectionAndSkipOne(self):
        self.settings.set('effortviewer', 'columnwidths', 'dict(subject=10)')
        self.settings.add_section('effortviewer2', 
            copyFromSection='effortviewer')
        self.assertEqual(dict(subject=10), 
            self.settings.getdict('effortviewer2', 'columnwidths'))

    def testSinglePercentage(self):
        # Prevent ValueError: invalid interpolation syntax in '%' at position 0:
        self.settings.set('effortviewer', 'searchfilterstring', '%')
        self.assertEqual('%', self.settings.get('effortviewer', 'searchfilterstring'))

    def testEmbeddedPercentage(self):
        # Prevent ValueError: invalid interpolation syntax in '%' at position 0
        self.settings.set('effortviewer', 'searchfilterstring', 'Bla%Bla')
        self.assertEqual('Bla%Bla', self.settings.get('effortviewer', 'searchfilterstring'))

    def testDoublePercentage(self):
        # Prevent ValueError: invalid interpolation syntax in '%' at position 0
        self.settings.set('effortviewer', 'searchfilterstring', '%%')
        self.assertEqual('%%', self.settings.get('effortviewer', 'searchfilterstring'))
        
    def testFixInvalidValuesFromOldIniFile(self):
        self.settings.set('feature', 'notifier', 'Native')
        self.assertEqual('Task Coach', self.settings.get('feature', 'notifier'))
        self.assertEqual('Task Coach', self.settings.getRawValue('feature', 'notifier'))
        

class SettingsIOTest(SettingsTestCase):
    def setUp(self):
        super(SettingsIOTest, self).setUp()
        self.fakeFile = StringIO.StringIO()

    def testSave(self):
        self.settings.write(self.fakeFile)
        self.fakeFile.seek(0)
        self.assertEqual('[%s]\n'%self.settings.sections()[0], 
            self.fakeFile.readline())

    def testRead(self):
        self.fakeFile.write('[testing]\n')
        self.fakeFile.seek(0)
        self.settings.readfp(self.fakeFile)
        self.failUnless(self.settings.has_section('testing'))
        
    def testIOErrorWhileSaving(self):
        def file_that_raises_ioerror(*args):  # pylint: disable=W0613,W0622
            raise IOError
        
        def showerror(*args, **kwargs):  # pylint: disable=W0613
            self.showerror_args = args  # pylint: disable=W0201
            
        settings = config.Settings()
        settings.save(showerror=showerror, file=file_that_raises_ioerror)
        self.failUnless(self.showerror_args)

    def testIOErrorWhileReading(self):
        class SettingsThatThrowsParsingError(config.Settings):
            def read(self, *args, **kwargs):  # pylint: disable=W0613
                self.remove_section('file')
                raise ConfigParser.ParsingError, 'Testing'
            
        self.failIf(SettingsThatThrowsParsingError().getboolean('file', 'inifileloaded'))
        
    def testFixOldColumnValues(self):
        section = 'prerequisiteviewerintaskeditor1'
        self.fakeFile.write("[%s]\ncolumns = ['dueDate']\ncolumnwidths = {'dueDate': 40}\n" % section)
        self.fakeFile.seek(0)
        self.settings.readfp(self.fakeFile)
        self.failUnless(['dueDateTime'], 
                        self.settings.getlist(section, 'columns'))
        self.assertEqual(dict(dueDateTime=40), 
                         self.settings.getdict(section, 'columnwidths'))


class SettingsObservableTest(SettingsTestCase):
    def setUp(self):
        super(SettingsObservableTest, self).setUp()
        self.events = []
        pub.subscribe(self.onEvent, 'settings.view.toolbar')
        
    def tearDown(self):
        pub.clearNotificationHandlers()

    def onEvent(self, value):
        self.events.append(value)
        
    def testChangingTheSettingCausesNotification(self):
        self.settings.settuple('view', 'toolbar', (16, 16))
        self.assertEqual((16, 16), self.events[0])
        
    def testChangingAnotherSettingDoesNotCauseANotification(self):
        self.settings.set('view', 'statusbar', 'True')
        self.failIf(self.events)


class UnicodeAwareConfigParserTest(test.TestCase):
    ''' The default Python ConfigParser does not deal with unicode. So we
        build a wrapper around ConfigParser that does. These are the unittests
        for UnicodeAwareConfigParser. '''
        
    def setUp(self):
        self.parser = config.settings.UnicodeAwareConfigParser()
        self.parser.add_section('section')
        self.iniFile = StringIO.StringIO()
        self.asciiValue = 'ascii'
        self.unicodeValue = u'√ÉÔøΩ√¢‚Ç¨¬¶√É≈Ω√Ç¬Ω√É≈Ω√Ç¬π√É≈Ω√Ç¬≥√É≈Ω√Ç¬ø√É≈Ω√Ç¬¥√É≈Ω√Ç¬∑'
        
    def testWriteAsciiValue(self):
        self.parser.set('section', 'setting', self.asciiValue)
        self.parser.write(self.iniFile)
        fileContents = self.iniFile.getvalue()
        self.assertEqual('[section]\nsetting = %s\n\n'%self.asciiValue, 
                         fileContents)
                
    def testWriteUnicodeValue(self):
        self.parser.set('section', 'setting', self.unicodeValue)
        self.parser.write(self.iniFile)
        fileContents = self.iniFile.getvalue()
        self.assertEqual('[section]\nsetting = %s\n\n' \
                         %self.unicodeValue.encode('utf-8'), fileContents)
    
    def testReadAsciiValue(self):
        iniFileContents = '[section]\nsetting = %s\n\n'%self.asciiValue
        self.iniFile.write(iniFileContents)
        self.iniFile.seek(0)
        self.parser.readfp(self.iniFile)
        self.assertEqual(self.asciiValue, self.parser.get('section', 'setting'))
        
    def testReadUnicodeValue(self):
        iniFileContents = '[section]\nsetting = %s\n\n' \
            %self.unicodeValue.encode('utf-8')
        self.iniFile.write(iniFileContents)
        self.iniFile.seek(0)
        self.parser.readfp(self.iniFile)
        self.assertEqual(self.unicodeValue, 
                         self.parser.get('section', 'setting'))
        

class SpecificSettingsTest(SettingsTestCase):
    def testDefaultWindowPosition(self):
        self.assertEqual('(-1, -1)', self.settings.get('window', 'position'))
        
    def testSetCurrentVersionAtSave(self):
        self.settings.set('version', 'current', '0.0')
        self.settings.save()
        self.assertEqual(meta.data.version, self.settings.get('version', 'current'))
            

class SettingsFileLocationTest(SettingsTestCase):
    def testDefaultSetting(self):
        self.assertEqual(False, self.settings.getboolean('file', 
                         'saveinifileinprogramdir'))

    def testPathWhenNotSavingIniFileInProgramDir(self):
        self.assertNotEqual(sys.argv[0], self.settings.path())
        
    def testPathWhenSavingIniFileInProgramDir(self):
        self.settings.setboolean('file', 'saveinifileinprogramdir', True)
        self.assertEqual(os.path.dirname(sys.argv[0]), self.settings.path())
        
    def testPathWhenSavingIniFileInProgramDirAndRunFromZipFile(self):
        self.settings.setboolean('file', 'saveinifileinprogramdir', True)
        sys.argv.insert(0, os.path.join('d:', 'TaskCoach', 'library.zip'))
        self.assertEqual(os.path.join('d:', 'TaskCoach'), self.settings.path())
        del sys.argv[0]
        
    def testSettingSaveIniFileInProgramDirToFalseRemovesIniFile(self):
        class SettingsUnderTest(config.Settings):
            def onSettingsFileLocationChanged(self, value):
                self.onSettingsFileLocationChangedCalled = value # pylint: disable=W0201
        settings = SettingsUnderTest(load=False)
        settings.setboolean('file', 'saveinifileinprogramdir', True)
        settings.setboolean('file', 'saveinifileinprogramdir', False)
        self.failIf(settings.onSettingsFileLocationChangedCalled)


class MinimumSettingsTest(SettingsTestCase):
    def testAtLeastOneTaskTreeListViewer(self):
        self.assertEqual(1, self.settings.getint('view', 'taskviewercount'))

    def testTwoTaskTreeListViewers(self):
        self.settings.set('view', 'taskviewercount', u'2')
        self.assertEqual(2, self.settings.getint('view', 'taskviewercount'))

    def testAtLeastOneTaskTreeListViewer_EvenWhenSetToZero(self):
        self.settings.set('view', 'taskviewercount', u'0')
        self.assertEqual(1, self.settings.getint('view', 'taskviewercount'))
        
        
class ApplicationOptionsTest(test.TestCase):
    def setUp(self):
        super(ApplicationOptionsTest, self).setUp()
        self.parser = config.ApplicationOptionParser()
        
    def parse(self, *args):
        return self.parser.parse_args(list(args))[0]
        
    def testUsage(self):
        self.assertEqual('%prog [options] [.tsk file]', self.parser.usage)
        
    def testLanguage(self):
        options = self.parse('-l', 'nl')
        self.assertEqual('nl', options.language)

    def testLanguageWhenNotChanged(self):
        options = self.parse()
        self.assertEqual(None, options.language)
        
    def testPoFile(self):
        options = self.parse('-p', 'test.po')
        self.assertEqual('test.po', options.pofile)

    def testIniFile(self):
        options = self.parse('-i', 'test.ini')
        self.assertEqual('test.ini', options.inifile)
        
    def testProfile(self):
        options = self.parse('--profile')
        self.failUnless(options.profile)
        



# -*- coding: utf-8 -*-

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''


from taskcoachlib import patterns
from taskcoachlib.domain import task, effort, date
from taskcoachlib.i18n import _
import base
import noteCommands


class SaveTaskStateMixin(base.SaveStateMixin, base.CompositeMixin):
    pass


class EffortCommand(base.BaseCommand):  # pylint: disable=W0223
    def stopTracking(self):
        self.stoppedEfforts = []  # pylint: disable=W0201
        for taskToStop in self.tasksToStopTracking():
            self.stoppedEfforts.extend(taskToStop.activeEfforts())
            taskToStop.stopTracking()

    def startTracking(self):
        for stoppedEffort in self.stoppedEfforts:
            stoppedEffort.setStop(date.DateTime.max)
    
    def tasksToStopTracking(self):
        return self.list
           
    def do_command(self):
        super(EffortCommand, self).do_command()
        self.stopTracking()
    
    def undo_command(self):
        super(EffortCommand, self).undo_command()
        self.startTracking()
    
    def redo_command(self):
        super(EffortCommand, self).redo_command()
        self.stopTracking()


class DragAndDropTaskCommand(base.OrderingDragAndDropCommand):
    plural_name = _('Drag and drop tasks')

    def getItemsToSave(self):
        toSave = super(DragAndDropTaskCommand, self).getItemsToSave()
        if self.part != 0:
            toSave.extend(self.getSiblings())
        return list(set(toSave)) # Because parent may have added siblings as well

    def do_command(self):
        if self.part == 0 or self.isOrdering():
            super(DragAndDropTaskCommand, self).do_command()
        else:
            if self.part == -1:
                # Up part. Add dropped items as prerequisites of dropped on item.
                self._itemToDropOn.addPrerequisites(self.items)
                self._itemToDropOn.addTaskAsDependencyOf(self.items)
            else:
                # Down. Add dropped on item as prerequisite of dropped items.
                for item in self.items:
                    item.addPrerequisites([self._itemToDropOn])
                    item.addTaskAsDependencyOf([self._itemToDropOn])

    def undo_command(self):
        if self.part == 0 or self.isOrdering():
            super(DragAndDropTaskCommand, self).undo_command()
        elif self.part == -1:
            self._itemToDropOn.removePrerequisites(self.items)
            self._itemToDropOn.removeTaskAsDependencyOf(self.items)
        else:
            for item in self.items:
                item.removePrerequisites([self._itemToDropOn])
                item.removeTaskAsDependencyOf([self._itemToDropOn])

    def redo_command(self):
        if self.part == 0 or self.isOrdering():
            super(DragAndDropTaskCommand, self).redo_command()
        elif self.part == -1:
            self._itemToDropOn.addPrerequisites(self.items)
            self._itemToDropOn.addTaskAsDependencyOf(self.items)
        else:
            for item in self.items:
                item.addPrerequisites([self._itemToDropOn])
                item.addTaskAsDependencyOf([self._itemToDropOn])


class DeleteTaskCommand(base.DeleteCommand, EffortCommand):
    plural_name = _('Delete tasks')
    singular_name = _('Delete task "%s"')

    def tasksToStopTracking(self):
        return self.items

    def do_command(self):
        super(DeleteTaskCommand, self).do_command()
        self.stopTracking()
        self.__removePrerequisites()
    
    def undo_command(self):
        super(DeleteTaskCommand, self).undo_command()
        self.startTracking()
        self.__restorePrerequisites()
        
    def redo_command(self):
        super(DeleteTaskCommand, self).redo_command()
        self.stopTracking()
        self.__removePrerequisites()
    
    def __removePrerequisites(self):
        self.__relationsToRestore = dict()  # pylint: disable=W0201
        for eachTask in self.items:
            prerequisites, dependencies = eachTask.prerequisites(), eachTask.dependencies()
            self.__relationsToRestore[eachTask] = prerequisites, dependencies
            eachTask.removeTaskAsDependencyOf(prerequisites)
            eachTask.removeTaskAsPrerequisiteOf(dependencies)
            eachTask.setPrerequisites([])
            eachTask.setDependencies([])

    def __restorePrerequisites(self):
        for eachTask, (prerequisites, dependencies) in self.__relationsToRestore.items():
            eachTask.addTaskAsDependencyOf(prerequisites)
            eachTask.addTaskAsPrerequisiteOf(dependencies)
            eachTask.setPrerequisites(prerequisites)
            eachTask.setDependencies(dependencies)
            

class NewTaskCommand(base.NewItemCommand):
    singular_name = _('New task')
    
    def __init__(self, *args, **kwargs):
        subject = kwargs.pop('subject', _('New task'))
        super(NewTaskCommand, self).__init__(*args, **kwargs)
        self.items = [task.Task(subject=subject, **kwargs)]

    @patterns.eventSource
    def do_command(self, event=None):
        super(NewTaskCommand, self).do_command(event=event)
        self.addDependenciesAndPrerequisites()
        
    @patterns.eventSource
    def undo_command(self, event=None):
        super(NewTaskCommand, self).undo_command(event=event)
        self.removeDependenciesAndPrerequisites()
        
    @patterns.eventSource
    def redo_command(self, event=None):
        super(NewTaskCommand, self).redo_command(event=event)
        self.addDependenciesAndPrerequisites()
        
    def addDependenciesAndPrerequisites(self):
        for eachTask in self.items:
            for prerequisite in eachTask.prerequisites():
                prerequisite.addDependencies([eachTask])
            for dependency in eachTask.dependencies():
                dependency.addPrerequisites([eachTask])

    def removeDependenciesAndPrerequisites(self):
        for eachTask in self.items:
            for prerequisite in eachTask.prerequisites():
                prerequisite.removeDependencies([eachTask])                                
            for dependency in eachTask.dependencies():
                dependency.removePrerequisites([eachTask])
                
                
class NewSubTaskCommand(base.NewSubItemCommand, SaveTaskStateMixin):
    plural_name = _('New subtasks')
    singular_name = _('New subtask of "%s"')
    # pylint: disable=E1101
    
    def __init__(self, *args, **kwargs):
        subject = kwargs.pop('subject', _('New subtask'))
        plannedStartDateTime = kwargs.pop('plannedStartDateTime', date.DateTime())
        dueDateTime = kwargs.pop('dueDateTime', date.DateTime())
        super(NewSubTaskCommand, self).__init__(*args, **kwargs)
        self.items = [parent.newChild(subject=subject, 
                                      plannedStartDateTime=max([parent.plannedStartDateTime(), plannedStartDateTime]),
                                      dueDateTime=min([parent.dueDateTime(), dueDateTime]), 
                                      **kwargs) for parent in self.items]
        self.saveStates(self.getTasksToSave())
        self.save_modification_datetimes()
    
    def getTasksToSave(self):
        # FIXME: can be simplified to: return self.getAncestors(self.items) ?
        parents = [item.parent() for item in self.items if item.parent()]
        return parents + self.getAncestors(parents)

    @patterns.eventSource
    def undo_command(self, event=None):
        super(NewSubTaskCommand, self).undo_command(event=event)
        self.undoStates(event=event)

    @patterns.eventSource
    def redo_command(self, event=None):
        super(NewSubTaskCommand, self).redo_command(event=event)
        self.redoStates(event=event)
                
                
class MarkCompletedCommand(base.SaveStateMixin, EffortCommand):
    plural_name = _('Mark tasks completed')
    singular_name = _('Mark "%s" completed')
    
    def __init__(self, *args, **kwargs):
        super(MarkCompletedCommand, self).__init__(*args, **kwargs)
        self.items = [item for item in self.items if item.completionDateTime() > date.Now()]
        itemsToSave = set([relative for item in self.items for relative in item.family()]) 
        self.saveStates(itemsToSave)

    def do_command(self):
        super(MarkCompletedCommand, self).do_command()
        for item in self.items:
            item.setCompletionDateTime(task.Task.suggestedCompletionDateTime())

    def undo_command(self):
        self.undoStates()
        super(MarkCompletedCommand, self).undo_command()

    def redo_command(self):
        self.redoStates()
        super(MarkCompletedCommand, self).redo_command()

    def tasksToStopTracking(self):
        return self.items                


class MarkActiveCommand(base.SaveStateMixin, base.BaseCommand):
    plural_name = _('Mark task active')
    singular_name = _('Mark "%s" active')
    
    def __init__(self, *args, **kwargs):
        super(MarkActiveCommand, self).__init__(*args, **kwargs)
        self.items = [item for item in self.items if item.actualStartDateTime() > date.Now() or item.completionDateTime() != date.DateTime()]
        itemsToSave = set([relative for item in self.items for relative in item.family()]) 
        self.saveStates(itemsToSave)

    def do_command(self):
        super(MarkActiveCommand, self).do_command()
        for item in self.items:
            item.setActualStartDateTime(task.Task.suggestedActualStartDateTime())
            item.setCompletionDateTime(date.DateTime())

    def undo_command(self):
        self.undoStates()
        super(MarkActiveCommand, self).undo_command()

    def redo_command(self):
        self.redoStates()
        super(MarkActiveCommand, self).redo_command()


class MarkInactiveCommand(base.SaveStateMixin, base.BaseCommand):
    plural_name = _('Mark task inactive')
    singular_name = _('Mark "%s" inactive')
    
    def __init__(self, *args, **kwargs):
        super(MarkInactiveCommand, self).__init__(*args, **kwargs)
        self.items = [item for item in self.items if item.actualStartDateTime() != date.DateTime() or item.completionDateTime() != date.DateTime()]
        itemsToSave = set([relative for item in self.items for relative in item.family()]) 
        self.saveStates(itemsToSave)

    def do_command(self):
        super(MarkInactiveCommand, self).do_command()
        for item in self.items:
            item.setActualStartDateTime(date.DateTime())
            item.setCompletionDateTime(date.DateTime())

    def undo_command(self):
        self.undoStates()
        super(MarkInactiveCommand, self).undo_command()

    def redo_command(self):
        self.redoStates()
        super(MarkInactiveCommand, self).redo_command()
    
                
class StartEffortCommand(EffortCommand):
    plural_name = _('Start tracking')
    singular_name = _('Start tracking "%s"')

    def __init__(self, *args, **kwargs):
        super(StartEffortCommand, self).__init__(*args, **kwargs)
        start = date.DateTime.now()
        self.efforts = [effort.Effort(item, start) for item in self.items]
        self.actualStartDateTimes = [(item.actualStartDateTime() if start < item.actualStartDateTime() else None) for item in self.items]
        
    def do_command(self):
        super(StartEffortCommand, self).do_command()
        self.addEfforts()
        
    def undo_command(self):
        self.removeEfforts()
        super(StartEffortCommand, self).undo_command()
        
    def redo_command(self):
        super(StartEffortCommand, self).redo_command()
        self.addEfforts()

    def addEfforts(self):
        for item, newEffort, currentActualStartDateTime in zip(self.items, self.efforts, self.actualStartDateTimes):
            item.addEffort(newEffort)
            if currentActualStartDateTime:
                item.setActualStartDateTime(newEffort.getStart())
            
    def removeEfforts(self):
        for item, newEffort, previousActualStartDateTime in zip(self.items, self.efforts, self.actualStartDateTimes):
            item.removeEffort(newEffort)
            if previousActualStartDateTime:
                item.setActualStartDateTime(previousActualStartDateTime)
            
        
class StopEffortCommand(EffortCommand):
    plural_name = _('Stop tracking')
    singular_name = _('Stop tracking "%s"')
                  
    def canDo(self):
        return True  # No selected items needed.
    
    def tasksToStopTracking(self):
        stoppable = lambda effort: effort.isBeingTracked() and not effort.isTotal()
        return set([effort.task() for effort in (self.items if self.items else self.list) if stoppable(effort)])  # pylint: disable=W0621 


class ExtremePriorityCommand(base.BaseCommand):  # pylint: disable=W0223
    delta = 'Subclass responsibility'
    
    def __init__(self, *args, **kwargs):
        super(ExtremePriorityCommand, self).__init__(*args, **kwargs)
        self.oldPriorities = [item.priority() for item in self.items]
        self.oldExtremePriority = self.getOldExtremePriority()
        
    def getOldExtremePriority(self):
        raise NotImplementedError  # pragma: no cover

    def setNewExtremePriority(self):
        newExtremePriority = self.oldExtremePriority + self.delta 
        for item in self.items:
            item.setPriority(newExtremePriority)

    def restorePriorities(self):
        for item, oldPriority in zip(self.items, self.oldPriorities):
            item.setPriority(oldPriority)

    def do_command(self):
        super(ExtremePriorityCommand, self).do_command()
        self.setNewExtremePriority()
        
    def undo_command(self):
        self.restorePriorities()
        super(ExtremePriorityCommand, self).undo_command()
        
    def redo_command(self):
        super(ExtremePriorityCommand, self).redo_command()
        self.setNewExtremePriority()


class MaxPriorityCommand(ExtremePriorityCommand):
    plural_name = _('Maximize priority')
    singular_name = _('Maximize priority of "%s"')
    
    delta = +1
    
    def getOldExtremePriority(self):
        return self.list.maxPriority()
    

class MinPriorityCommand(ExtremePriorityCommand):
    plural_name = _('Minimize priority')
    singular_name = _('Minimize priority of "%s"')
    
    delta = -1
                    
    def getOldExtremePriority(self):
        return self.list.minPriority()
    

class ChangePriorityCommand(base.BaseCommand):  # pylint: disable=W0223
    delta = 'Subclass responsibility'
    
    def changePriorities(self, delta):
        for item in self.items:
            item.setPriority(item.priority() + delta)

    def do_command(self):
        super(ChangePriorityCommand, self).do_command()
        self.changePriorities(self.delta)

    def undo_command(self):
        self.changePriorities(-self.delta)
        super(ChangePriorityCommand, self).undo_command()

    def redo_command(self):
        super(ChangePriorityCommand, self).redo_command()
        self.changePriorities(self.delta)


class IncPriorityCommand(ChangePriorityCommand):
    plural_name = _('Increase priority')
    singular_name = _('Increase priority of "%s"')
    delta = +1


class DecPriorityCommand(ChangePriorityCommand):
    plural_name = _('Decrease priority')
    singular_name = _('Decrease priority of "%s"')
    delta = -1
    

class EditPriorityCommand(base.BaseCommand):
    plural_name = _('Change priority')
    singular_name = _('Change priority of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newPriority = kwargs.pop('newValue')
        super(EditPriorityCommand, self).__init__(*args, **kwargs)
        self.__oldPriorities = [item.priority() for item in self.items]

    def do_command(self):
        super(EditPriorityCommand, self).do_command()
        for item in self.items:
            item.setPriority(self.__newPriority)

    def undo_command(self):
        super(EditPriorityCommand, self).undo_command()
        for item, oldPriority in zip(self.items, self.__oldPriorities):
            item.setPriority(oldPriority)

    def redo_command(self):
        self.do_command()
    
    
class AddTaskNoteCommand(noteCommands.AddNoteCommand):
    plural_name = _('Add note to tasks')


class EditDateTimeCommand(base.BaseCommand):
    def __init__(self, *args, **kwargs):
        self._newDateTime = kwargs.pop('newValue')
        super(EditDateTimeCommand, self).__init__(*args, **kwargs)
        self.__oldDateTimes = [self.getDateTime(item) for item in self.items]
        familyMembers = set()
        for item in self.items:
            for familyMember in item.family():
                if familyMember not in self.items:
                    familyMembers.add(familyMember)
        self.__oldFamilyMemberDateTimes = [(familyMember, self.getDateTime(familyMember)) for familyMember in familyMembers]                

    @staticmethod
    def getDateTime(item):
        raise NotImplementedError  # pragma: no cover

    @staticmethod
    def setDateTime(item, newDateTime):
        raise NotImplementedError  # pragma: no cover
    
    def do_command(self):
        super(EditDateTimeCommand, self).do_command()
        for item in self.items:
            self.setDateTime(item, self._newDateTime)

    def undo_command(self):
        super(EditDateTimeCommand, self).undo_command()
        for item, oldDateTime in zip(self.items, self.__oldDateTimes):
            self.setDateTime(item, oldDateTime)
        for familyMember, oldDateTime in self.__oldFamilyMemberDateTimes:
            self.setDateTime(familyMember, oldDateTime)
    
    def redo_command(self):
        self.do_command()
    

class EditPeriodDateTimeCommand(EditDateTimeCommand):
    ''' Base for date/time commands that also may have to adjust the other
        end of the period. E.g., where changing the planned start date should 
        also change the due date to keep the period the same length. '''
    
    def __init__(self, *args, **kwargs):            
        self.__keep_delta = kwargs.pop('keep_delta', False)
        super(EditPeriodDateTimeCommand, self).__init__(*args, **kwargs)

    def do_command(self):
        self.__adjustOtherDateTime(direction=1)
        super(EditPeriodDateTimeCommand, self).do_command()

    def undo_command(self):
        super(EditPeriodDateTimeCommand, self).undo_command()
        self.__adjustOtherDateTime(direction=-1)

    def __adjustOtherDateTime(self, direction):
        for item in self.items:
            if self.__shouldAdjustItem(item):
                delta = direction * (self._newDateTime - self.getDateTime(item))
                newOtherDateTime = self.getOtherDateTime(item) + delta
                self.setOtherDateTime(item, newOtherDateTime)

    def __shouldAdjustItem(self, item):
        ''' Determine whether the other date/time of the item should be
            adjusted. '''
        return self.__keep_delta and date.DateTime() not in (self._newDateTime, 
                                                             item.plannedStartDateTime(),
                                                             item.dueDateTime())

    @staticmethod
    def getOtherDateTime(item):
        ''' Gets the date/time that represents the other end of the period. '''
        raise NotImplementedError  # pragma: no cover
    
    @staticmethod
    def setOtherDateTime(item, newDateTime):
        ''' Set the date/time that represents the other end of the period. '''
        raise NotImplementedError  # pragma: no cover
    
    
class EditPlannedStartDateTimeCommand(EditPeriodDateTimeCommand):
    plural_name = _('Change planned start date')
    singular_name = _('Change planned start date of "%s"')
    
    @staticmethod
    def getDateTime(item):
        return item.plannedStartDateTime()
    
    @staticmethod
    def setDateTime(item, dateTime):
        item.setPlannedStartDateTime(dateTime)
        
    @staticmethod
    def getOtherDateTime(item):
        return item.dueDateTime()
    
    @staticmethod
    def setOtherDateTime(item, dateTime):
        item.setDueDateTime(dateTime)


class EditDueDateTimeCommand(EditPeriodDateTimeCommand):
    plural_name = _('Change due date')
    singular_name = _('Change due date of "%s"')
    
    @staticmethod
    def getDateTime(item):
        return item.dueDateTime()
    
    @staticmethod
    def setDateTime(item, dateTime):
        item.setDueDateTime(dateTime)
        
    @staticmethod
    def getOtherDateTime(item):
        return item.plannedStartDateTime()

    @staticmethod
    def setOtherDateTime(item, dateTime):
        item.setPlannedStartDateTime(dateTime)
 

class EditActualStartDateTimeCommand(EditPeriodDateTimeCommand):
    plural_name = _('Change actual start date')
    singular_name = _('Change actual start date of "%s"')
    
    @staticmethod
    def getDateTime(item):
        return item.actualStartDateTime()
    
    @staticmethod
    def setDateTime(item, dateTime):
        item.setActualStartDateTime(dateTime)
        
    @staticmethod
    def getOtherDateTime(item):
        return item.completionDateTime()
    
    @staticmethod
    def setOtherDateTime(item, dateTime):
        item.setCompletionDateTime(dateTime)
               

class EditCompletionDateTimeCommand(EditDateTimeCommand, EffortCommand):
    plural_name = _('Change completion date')
    singular_name = _('Change completion date of "%s"')
                    
    @staticmethod
    def getDateTime(item):
        return item.completionDateTime()
    
    @staticmethod
    def setDateTime(item, dateTime):
        item.setCompletionDateTime(dateTime)

    @staticmethod
    def getOtherDateTime(item):
        return item.actualStartDateTime()
    
    @staticmethod
    def setOtherDateTime(item, dateTime):
        item.setActualStartDateTime(dateTime)

    def tasksToStopTracking(self):
        return self.items
    
    
class EditReminderDateTimeCommand(EditDateTimeCommand):
    plural_name = _('Change reminder dates/times')
    singular_name = _('Change reminder date/time of "%s"')
    
    @staticmethod
    def getDateTime(item):
        return item.reminder()
    
    @staticmethod
    def setDateTime(item, dateTime):
        item.setReminder(dateTime)

        
class EditRecurrenceCommand(base.BaseCommand):
    plural_name = _('Change recurrences')
    singular_name = _('Change recurrence of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newRecurrence = kwargs.pop('newValue')
        super(EditRecurrenceCommand, self).__init__(*args, **kwargs)
        self.__oldRecurrences = [item.recurrence() for item in self.items]

    def do_command(self):
        super(EditRecurrenceCommand, self).do_command()
        for item in self.items:
            item.setRecurrence(self.__newRecurrence)

    def undo_command(self):
        super(EditRecurrenceCommand, self).undo_command()
        for item, oldRecurrence in zip(self.items, self.__oldRecurrences):
            item.setRecurrence(oldRecurrence)

    def redo_command(self):
        self.do_command()
        
    
class EditPercentageCompleteCommand(base.SaveStateMixin, EffortCommand):
    plurar_name = ('Change percentages complete')
    singular_name = _('Change percentage complete of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newPercentage = kwargs.pop('newValue')
        super(EditPercentageCompleteCommand, self).__init__(*args, **kwargs)
        itemsToSave = set([relative for item in self.items for relative in item.family()]) 
        self.saveStates(itemsToSave)
        
    def do_command(self):
        super(EditPercentageCompleteCommand, self).do_command()
        for item in self.items:
            item.setPercentageComplete(self.__newPercentage)
            
    def undo_command(self):
        self.undoStates()
        super(EditPercentageCompleteCommand, self).undo_command()
        
    def redo_command(self):
        self.redoStates()
        super(EditPercentageCompleteCommand, self).redo_command()

    def tasksToStopTracking(self):
        return self.items if self.__newPercentage == 100 else []
        

class EditShouldMarkCompletedCommand(base.BaseCommand):
    plural_name = _('Change when tasks are marked completed')
    singular_name = _('Change when "%s" is marked completed')
    
    def __init__(self, *args, **kwargs):
        self.__newShouldMarkCompleted = kwargs.pop('newValue')
        super(EditShouldMarkCompletedCommand, self).__init__(*args, **kwargs)
        self.__oldShouldMarkCompleted = [item.shouldMarkCompletedWhenAllChildrenCompleted() for item in self.items]
        
    def do_command(self):
        super(EditShouldMarkCompletedCommand, self).do_command()
        for item in self.items:
            item.setShouldMarkCompletedWhenAllChildrenCompleted(self.__newShouldMarkCompleted)
            
    def undo_command(self):
        super(EditShouldMarkCompletedCommand, self).undo_command()
        for item, oldShouldMarkCompleted in zip(self.items, self.__oldShouldMarkCompleted):
            item.setShouldMarkCompletedWhenAllChildrenCompleted(oldShouldMarkCompleted)
            
    def redo_command(self):
        self.do_command()
        

class EditBudgetCommand(base.BaseCommand):
    plural_name = _('Change budgets')
    singular_name = _('Change budget of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newBudget = kwargs.pop('newValue') 
        super(EditBudgetCommand, self).__init__(*args, **kwargs)
        self.__oldBudgets = [item.budget() for item in self.items]
        
    def do_command(self):
        super(EditBudgetCommand, self).do_command()
        for item in self.items:
            item.setBudget(self.__newBudget)
            
    def undo_command(self):
        super(EditBudgetCommand, self).undo_command()
        for item, oldBudget in zip(self.items, self.__oldBudgets):
            item.setBudget(oldBudget)
            
    def redo_command(self):
        self.do_command()
            
            
class EditHourlyFeeCommand(base.BaseCommand):
    plural_name = _('Change hourly fees')
    singular_name = _('Change hourly fee of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newHourlyFee = kwargs.pop('newValue')
        super(EditHourlyFeeCommand, self).__init__(*args, **kwargs)
        self.__oldHourlyFees = [item.hourlyFee() for item in self.items]
        
    def do_command(self):
        super(EditHourlyFeeCommand, self).do_command()
        for item in self.items:
            item.setHourlyFee(self.__newHourlyFee)
            
    def undo_command(self):
        super(EditHourlyFeeCommand, self).undo_command()
        for item, oldHourlyFee in zip(self.items, self.__oldHourlyFees):
            item.setHourlyFee(oldHourlyFee)
            
    def redo_command(self):
        self.do_command()


class EditFixedFeeCommand(base.BaseCommand):
    plural_name = _('Change fixed fees')
    singular_name = _('Change fixed fee of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__newFixedFee = kwargs.pop('newValue')
        super(EditFixedFeeCommand, self).__init__(*args, **kwargs)
        self.__oldFixedFees = [item.fixedFee() for item in self.items]
        
    def do_command(self):
        super(EditFixedFeeCommand, self).do_command()
        for item in self.items:
            item.setFixedFee(self.__newFixedFee)
            
    def undo_command(self):
        super(EditFixedFeeCommand, self).undo_command()
        for item, oldFixedFee in zip(self.items, self.__oldFixedFees):
            item.setFixedFee(oldFixedFee)
            
    def redo_command(self):
        self.do_command()
            
            
class TogglePrerequisiteCommand(base.BaseCommand):
    plural_name = _('Toggle prerequisite')
    singular_name = _('Toggle prerequisite of "%s"')
    
    def __init__(self, *args, **kwargs):
        self.__checkedPrerequisites = set(kwargs.pop('checkedPrerequisites'))
        self.__uncheckedPrerequisites = set(kwargs.pop('uncheckedPrerequisites'))
        super(TogglePrerequisiteCommand, self).__init__(*args, **kwargs)
    
    def do_command(self):
        super(TogglePrerequisiteCommand, self).do_command()
        for item in self.items:
            item.addPrerequisites(self.__checkedPrerequisites)
            item.addTaskAsDependencyOf(self.__checkedPrerequisites)
            item.removePrerequisites(self.__uncheckedPrerequisites)
            item.removeTaskAsDependencyOf(self.__uncheckedPrerequisites)

    def undo_command(self):
        super(TogglePrerequisiteCommand, self).undo_command()
        for item in self.items:
            item.removePrerequisites(self.__checkedPrerequisites)
            item.removeTaskAsDependencyOf(self.__checkedPrerequisites)
            item.addPrerequisites(self.__uncheckedPrerequisites)
            item.addTaskAsDependencyOf(self.__uncheckedPrerequisites)

    def redo_command(self):
        self.do_command()



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx, os, sys, imp, tempfile, locale, gettext
from taskcoachlib import patterns, operating_system
import po2dict


class Translator:
    __metaclass__ = patterns.Singleton
    
    def __init__(self, language):
        load = self._loadPoFile if language.endswith('.po') else self._loadModule
        module, language = load(language) 
        self._installModule(module)
        self._setLocale(language)

    def _loadPoFile(self, poFilename):
        ''' Load the translation from a .po file by creating a python 
            module with po2dict and them importing that module. ''' 
        language = self._languageFromPoFilename(poFilename)
        pyFilename = self._tmpPyFilename()
        po2dict.make(poFilename, pyFilename)
        module = imp.load_source(language, pyFilename)
        os.remove(pyFilename)
        return module, language
    
    def _tmpPyFilename(self):
        ''' Return a filename of a (closed) temporary .py file. '''
        tmpFile = tempfile.NamedTemporaryFile(suffix='.py')
        pyFilename = tmpFile.name
        tmpFile.close()
        return pyFilename

    def _loadModule(self, language):
        ''' Load the translation from a python module that has been 
            created from a .po file with po2dict before. '''
        for moduleName in self._localeStrings(language):
            try:
                module = __import__(moduleName, globals())
                break
            except ImportError:
                module = None
        return module, language

    def _installModule(self, module):
        ''' Make the module's translation dictionary and encoding available. '''
        # pylint: disable=W0201
        if module:
            self.__language = module.dict
            self.__encoding = module.encoding

    def _setLocale(self, language):
        ''' Try to set the locale, trying possibly multiple localeStrings. '''
        if not operating_system.isGTK():
            locale.setlocale(locale.LC_ALL, '')
        # Set the wxPython locale:
        for localeString in self._localeStrings(language):
            languageInfo = wx.Locale.FindLanguageInfo(localeString)
            if languageInfo:
                self.__locale = wx.Locale(languageInfo.Language) # pylint: disable=W0201
                # Add the wxWidgets message catalog. This is really only for 
                # py2exe'ified versions, but it doesn't seem to hurt on other
                # platforms...
                localeDir = os.path.join(wx.StandardPaths_Get().GetResourcesDir(), 'locale')
                self.__locale.AddCatalogLookupPathPrefix(localeDir)
                self.__locale.AddCatalog('wxstd')
                break
        if operating_system.isGTK():
            try:
                locale.setlocale(locale.LC_ALL, '')
            except locale.Error:
                # Mmmh. wx will display a message box later, so don't do anything.
                pass
        self._fixBrokenLocales()
            
    def _fixBrokenLocales(self):
        current_language = locale.getlocale(locale.LC_TIME)[0]
        if current_language and '_NO' in current_language:
            # nb_BO and ny_NO cause crashes in the wx.DatePicker. Set the
            # time part of the locale to some other locale. Since we don't
            # know which ones are available we try a few. First we try the
            # default locale of the user (''). It's probably *_NO, but it 
            # might be some other language so we try just in case. Then we try 
            # English (GB) so the user at least gets a European date and time 
            # format if that works. If all else fails we use the default 
            # 'C' locale.
            for lang in ['', 'en_GB.utf8', 'C']:
                try:
                    locale.setlocale(locale.LC_TIME, lang)
                except locale.Error:
                    continue
                current_language = locale.getlocale(locale.LC_TIME)[0]
                if current_language and '_NO' in current_language:
                    continue
                else: 
                    break

    def _localeStrings(self, language):
        ''' Extract language and language_country from language if possible. '''
        localeStrings = []
        if language:
            localeStrings.append(language)
            if '_' in language:
                localeStrings.append(language.split('_')[0])
        return localeStrings
    
    def _languageFromPoFilename(self, poFilename):
        return os.path.splitext(os.path.basename(poFilename))[0]
        
    def translate(self, string):
        ''' Look up string in the current language dictionary. Return the
            passed string if no language dictionary is available or if the
            dictionary doesn't contain the string. '''
        try:
            return self.__language[string].decode(self.__encoding)
        except (AttributeError, KeyError):
            return string

    
def currentLanguageIsRightToLeft():
    return wx.GetApp().GetLayoutDirection() == wx.Layout_RightToLeft       

def translate(string):
    return Translator().translate(string)

_ = translate # This prevents a warning from pygettext.py

# Inject into builtins for 3rdparty packages
import __builtin__
__builtin__.__dict__['_'] = _



#!/usr/bin/env python

'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wxversion
wxversion.select(["2.8-unicode", "3.0"], optionsRequired=True)

import sys, unittest, os, time, wx, logging
projectRoot = os.path.abspath('..')
if projectRoot not in sys.path:
    sys.path.insert(0, projectRoot)

from taskcoachlib.notify import AbstractNotifier


def skipOnPlatform(*platforms):
    ''' Decorator for unit tests that are to be skipped on specific 
        platforms. '''
    def wrapper(func):
        if wx.Platform in platforms:
            return lambda self, *args, **kwargs: self.skipTest('platform is %s' % wx.Platform)
        return func
    return wrapper


def skipOnTwistedVersions(*versions):
    ''' Decorator for unit tests that are to be skipped on specific
        versions of Twisted. Versions are strings. The test is
        skipped if the current Twisted version string is prefixed by any
        of the specified ones. '''
    def wrapper(func):
        import twisted
        if any([twisted.version.short().startswith(version) for version in versions]):
            return lambda self, *args, **kwargs: self.skipTest('Twisted version is %s' % twisted.version.short())
        return func
    return wrapper


class TestCase(unittest.TestCase, object):
    def assertEqualLists(self, expectedList, actualList):
        self.assertEqual(len(expectedList), len(actualList))
        for item in expectedList:
            self.failUnless(item in actualList)
            
    def registerObserver(self, eventType, eventSource=None):
        if not hasattr(self, 'events'):
            self.events = []  # pylint: disable=W0201
        from taskcoachlib import patterns  # pylint: disable=W0404
        patterns.Publisher().registerObserver(self.onEvent, eventType=eventType,
                                              eventSource=eventSource)
        
    def onEvent(self, event):
        self.events.append(event)

    def setUp(self):
        AbstractNotifier.disableNotifications()

    def tearDown(self):
        # pylint: disable=W0404
        # Prevent processing of pending events after the test has finished:
        wx.GetApp().Disconnect(wx.ID_ANY) 
        from taskcoachlib import patterns
        patterns.Publisher().clear()
        patterns.CommandHistory().clear()
        patterns.NumberedInstances.count = dict()
        from taskcoachlib.domain import date
        date.Scheduler().shutdown()
        date.Scheduler.deleteInstance()
        if hasattr(self, 'events'):
            del self.events
        from taskcoachlib.thirdparty.pubsub import pub
        pub.unsubAll()
        super(TestCase, self).tearDown()


class TestCaseFrame(wx.Frame):
    def __init__(self):
        super(TestCaseFrame, self).__init__(None, wx.ID_ANY, 'Frame')
        self.toolbarPerspective = ''

    def getToolBarPerspective(self):
        return self.toolbarPerspective

    def AddBalloonTip(self, *args, **kwargs):
        pass


class wxTestCase(TestCase):
    # pylint: disable=W0404
    app = wx.App(0)
    frame = TestCaseFrame()
    from taskcoachlib import i18n
    i18n.Translator('en_US')
    from taskcoachlib import gui
    gui.init()

    def tearDown(self):
        super(wxTestCase, self).tearDown()
        self.frame.DestroyChildren() # Clean up GDI objects on Windows


class TestResultWithTimings(unittest._TextTestResult): # pylint: disable=W0212
    def __init__(self, *args, **kwargs):
        super(TestResultWithTimings, self).__init__(*args, **kwargs)
        self._timings = {}

    def startTest(self, test):
        super(TestResultWithTimings, self).startTest(test)
        self._timings[test] = time.time()
        
    def stopTest(self, test):
        super(TestResultWithTimings, self).stopTest(test)
        self._timings[test] = time.time() - self._timings[test]


class TextTestRunnerWithTimings(unittest.TextTestRunner):
    def __init__(self, nrTestsToReport, timeTests=False, *args, **kwargs):
        super(TextTestRunnerWithTimings, self).__init__(*args, **kwargs)
        self._timeTests = timeTests
        self._nrTestsToReport = nrTestsToReport

    def _makeResult(self):
        return TestResultWithTimings(self.stream, self.descriptions, 
            self.verbosity)

    def run(self, *args, **kwargs): # pylint: disable=W0221
        result = super(TextTestRunnerWithTimings, self).run(*args, **kwargs)
        if self._timeTests:
            sortableTimings = [(timing, test) for test, timing in result._timings.items()] # pylint: disable=W0212
            sortableTimings.sort(reverse=True)
            print '\n%d slowest tests:'%self._nrTestsToReport
            for timing, test in sortableTimings[:self._nrTestsToReport]:
                print '%s (%.2f)'%(test, timing)
        return result


class AllTests(unittest.TestSuite):
    def __init__(self, options, testFiles=None):
        super(AllTests, self).__init__()
        self._options = options 
        self.loadAllTests(testFiles or [])

    def filenameToModuleName(self, filename):
        if filename == os.path.abspath(filename):
            # Strip current working directory to get the relative path:
            filename = filename[len(os.getcwd() + os.sep):]
        module = filename.replace(os.sep, '.')
        module = module.replace('/', '.')  
        return module[:-3] # strip '.py'

    def loadAllTests(self, testFiles):
        testloader = unittest.TestLoader()
        if not testFiles:
            if self._options.unittests:
                testFiles.extend(self.getTestFilesFromDir('unittests'))
            if self._options.integrationtests:
                testFiles.extend(self.getTestFilesFromDir('integrationtests'))
            if self._options.languagetests:
                testFiles.extend(self.getTestFilesFromDir('languagetests'))
            if self._options.releasetests:
                testFiles.extend(self.getTestFilesFromDir('releasetests'))
            if self._options.disttests:
                path = os.path.join('disttests', sys.platform)
                if os.path.exists(path):
                    testFiles.extend(self.getTestFilesFromDir(path))
                else:
                    print 'WARNING: no disttest for your platform (%s)' % sys.platform
        for filename in testFiles:
            moduleName = self.filenameToModuleName(filename)
            # Importing the module is not strictly necessary because
            # loadTestsFromName will do that too as a side effect. But if the 
            # test module contains errors our import will raise an exception
            # while loadTestsFromName ignores exceptions when importing from 
            # modules.
            __import__(moduleName)
            suite = testloader.loadTestsFromName(moduleName)
            self.addTests(suite._tests) # pylint: disable=W0212
   
    def runTests(self):       
        testrunner = TextTestRunnerWithTimings(
            verbosity=self._options.verbosity,
            timeTests=self._options.time,
            nrTestsToReport=self._options.time_reports)
        return testrunner.run(self)

    @staticmethod
    def getPyFilesFromDir(directory):
        return AllTests.getFilesFromDir(directory, '.py')

    @staticmethod
    def getTestFilesFromDir(directory):
        return AllTests.getFilesFromDir(directory, 'Test.py')

    @staticmethod
    def getFilesFromDir(directory, extension):
        result = []
        for root, dirs, filenames in os.walk(directory): # pylint: disable=W0612
            result.extend([os.path.join(root, filename) for filename in filenames \
                           if filename.endswith(extension)])
        return result


from taskcoachlib import config
class TestOptionParser(config.OptionParser):
    def __init__(self):
        super(TestOptionParser, self).__init__(usage='usage: %prog [options] [testfiles]')

    def testoutputOptionGroup(self):
        testoutput = config.OptionGroup(self, 'Test output',
            'Options to determine the amount of output while running the '
            'tests.')
        testoutput.add_option('-q', '--quiet', action='store_const', default=1,
            const=0, dest='verbosity', help='show only the final test result')
        testoutput.add_option('--progress', action='store_const', const=1,
            dest='verbosity', help='show progress [default]')
        testoutput.add_option('-v', '--verbose', action='store_const',
            const=2, dest='verbosity', help='show all tests')
        testoutput.add_option('-t', '--time', default=False, 
            action='store_true', 
            help='time the tests and report the slowest tests')
        testoutput.add_option('--time-reports', default=10, type='int',
            help='the number of slow tests to report [%default]')
        return testoutput

    def profileOptionGroup(self):
        profile = config.OptionGroup(self, 'Profiling', 
            'Options to profile the tests to see what test code or production '
            'code is taking the most time.')
        profile.add_option('-p', '--profile', default=False, 
            action='store_true', help='profile the running of all the tests')
        profile.add_option('-r', '--report-only', dest='profile_report_only', 
            action='store_true', default=False,
            help="don't make a new profile, report only on the last profile")
        profile.add_option('-s', '--sort', dest='profile_sort', 
            action='append', default=[],
            help="sort key to be used for reporting the profile data. "
            "Possible sort keys are: 'calls', 'cumulative' [default], "
            "'file', 'line', 'module', 'name', 'nfl', 'pcalls', 'stdname', "
            "and 'time'. This option may be repeated")
        profile.add_option('--callers', dest='profile_callers',
            default=False, action='store_true', help='print callers')
        profile.add_option('--callees', dest='profile_callees',
            default=False, action='store_true', help='print callees')
        profile.add_option('-l', '--limit', dest='profile_limit', default=50, 
            type="int", help="limit the number of calls to show in the "
            "profile reports [%default]")
        profile.add_option('--regex', dest='profile_regex',
            help='Regular expression to limit the functions shown in the '
           'profile reports')
        return profile

    def testselectionOptionGroup(self):
        testselection = config.OptionGroup(self, 'Test selection',
            'Options to determine which tests to run.')
        
        description = dict(dist='the platform-specific package', all='all')

        def helpText(selection):
            return 'run %s tests'%description.get(selection, 'the %s'%selection) + \
                   (' [default]' if selection == 'unit' else '')
        
        for selection in 'unit', 'integration', 'language', 'release', 'dist', 'all':
            testselection.add_option('--%stests'%selection, default=False,
                action='store_true', help=helpText(selection))

        return testselection

    def parse_args(self): # pylint: disable=W0221
        options, args = super(TestOptionParser, self).parse_args()
        if options.profile_report_only:
            options.profile = True
        if not options.profile_sort:
            options.profile_sort.append('cumulative')
        if not (options.unittests or options.integrationtests or \
                options.languagetests or options.releasetests or \
                options.disttests or options.alltests):
            options.unittests = True # the default option
        if options.alltests:
            options.unittests = True
            options.integrationtests = True
            options.languagetests = True
            options.releasetests = True
            options.disttests = True
        return options, args


class TestProfiler:
    def __init__(self, options, logfile='.profile'):
        self._logfile = logfile
        self._options = options

    def reportLastRun(self):
        import pstats # pylint: disable=W0404
        stats = pstats.Stats(self._logfile)
        stats.strip_dirs()
        for sortKey in self._options.profile_sort:
            stats.sort_stats(sortKey)
            stats.print_stats(self._options.profile_regex, 
                self._options.profile_limit)
        if self._options.profile_callers:
            stats.print_callers()
        if self._options.profile_callees:
            stats.print_callees()

    def run(self, tests, command='runTests'):
        if self._options.profile_report_only or self.profile(tests, command):
            self.reportLastRun()

    def profile(self, tests, command): # pylint: disable=W0613
        import cProfile # pylint: disable=W0404
        _locals = dict(locals())
        cProfile.runctx('result = tests.%s()'%command, globals(), _locals,
            filename=self._logfile)
        result = _locals['result']
        if not result.wasSuccessful():
            self.cleanup()
        return result.wasSuccessful()
            
    def cleanup(self):
        os.remove(self._logfile)

    
if __name__ == '__main__':
    logging.basicConfig()
    theOptions, theTestFiles = TestOptionParser().parse_args()
    allTests = AllTests(theOptions, theTestFiles)
    if theOptions.profile:
        TestProfiler(theOptions).run(allTests)
    else:
        theResult = allTests.runTests()
        if not theResult.wasSuccessful():
            sys.exit(1)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import data
import threading
import urllib2
import sys
import traceback
 

class VersionChecker(threading.Thread):
    def __init__(self, settings, verbose=False):
        self.settings = settings
        self.verbose = verbose
        super(VersionChecker, self).__init__()
        
    def _set_daemon(self):
        return True  # Don't block application exit
        
    def run(self):
        from taskcoachlib.gui.dialog import version
        try:
            latestVersionString = self.getLatestVersion()
            latestVersion = self.tupleVersion(latestVersionString)
            lastVersionNotified = self.tupleVersion(self.getLastVersionNotified())
            currentVersion = self.tupleVersion(data.version)
        except:
            if self.verbose:
                self.notifyUser(version.NoVersionDialog, 
                                message=''.join(traceback.format_exception_only(sys.exc_type, sys.exc_value)))
        else:
            if latestVersion < currentVersion and self.verbose:
                self.notifyUser(version.PrereleaseVersionDialog, latestVersionString)
            elif latestVersion == currentVersion and self.verbose:
                self.notifyUser(version.VersionUpToDateDialog, latestVersionString)
            elif latestVersion > currentVersion and (self.verbose or latestVersion > lastVersionNotified):
                self.setLastVersionNotified(latestVersionString)
                self.notifyUser(version.NewVersionDialog, latestVersionString)
            
    def getLatestVersion(self):
        versionText = self.parseVersionFile(self.retrieveVersionFile())
        return versionText.strip()

    def notifyUser(self, dialog, latestVersion='', message=''):
        # Must use CallAfter because this is a non-GUI thread
        # Import wx here so it isn't a build dependency
        import wx
        wx.CallAfter(self.showDialog, dialog, latestVersion, message)

    def showDialog(self, VersionDialog, latestVersion, message=''):
        import wx
        dialog = VersionDialog(wx.GetApp().GetTopWindow(), 
                               version=latestVersion, message=message, 
                               settings=self.settings)
        dialog.Show()
        return dialog
    
    def getLastVersionNotified(self):
        return self.settings.get('version', 'notified')
    
    def setLastVersionNotified(self, lastVersionNotifiedString):
        self.settings.set('version', 'notified', lastVersionNotifiedString)

    @staticmethod
    def parseVersionFile(versionFile):
        return versionFile.readline()

    @staticmethod
    def retrieveVersionFile():
        return urllib2.urlopen(data.version_url)

    @staticmethod
    def tupleVersion(versionString):
        return tuple(int(i) for i in versionString.split('.'))



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import sys, os, struct

_BINBASE = os.path.join(os.path.split(__file__)[0], '..', 'bin.in')

if len(struct.pack('L', 0)) == 8:
    arch = 'IA64'
else:
    arch = 'IA32'

if sys.platform == 'linux2':
    # The user should install the binary packages
    pass
elif sys.platform == 'darwin':
    sys.path.insert(0, os.path.join(_BINBASE, 'macos', arch))
else:
    sys.path.insert(0, os.path.join(_BINBASE, 'windows', 'py%s' % ''.join(map(str, sys.version_info[:2]))))

from _pysyncml import *



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib.i18n import _
from taskcoachlib import render
from taskcoachlib.domain import date
import StringIO


def extendedWithAncestors(selection):
    extendedSelection = selection[:]
    for item in selection:
        for ancestor in item.ancestors():
            if ancestor not in extendedSelection:
                extendedSelection.append(ancestor)
    return extendedSelection


class RowBuilder(object):
    dateAndTimeColumnHeaders = dict(actualStartDateTime=[_('Actual start date'), _('Actual start time')],
                                    plannedStartDateTime=[_('Planned start date'), _('Planned start time')],
                                    dueDateTime=[_('Due date'), _('Due time')],
                                    completionDateTime=[_('Completion date'), _('Completion time')],
                                    reminder=[_('Reminder date'), _('Reminder time')],
                                    creationDateTime=[_('Creation date'), _('Creation time')],
                                    period=[_('Period start date'), _('Period start time'), _('Period end date'), _('Period end time')])
    
    def __init__(self, columns, isTree, separateDateAndTimeColumns):
        self.__columns = columns
        self.__separateDateAndTimeColumns = separateDateAndTimeColumns
        if isTree:
            self.indent = lambda item: ' ' * len(item.ancestors())
        else:
            self.indent = lambda item: ''
        
    def headerRow(self):
        headers = []
        for column in self.__columns:
            if self.shouldSplitDateAndTime(column):
                headers.extend(self.dateAndTimeColumnHeaders[column.name()])
            else:
                headers.append(column.header())
        return headers
    
    def itemRow(self, item):
        row = []
        for column in self.__columns:
            if self.shouldSplitDateAndTime(column):
                row.extend(self.splitDateAndTime(column, item))
            elif column.name() == 'notes':
                def renderNotes(notes, indent=0):
                    bf = StringIO.StringIO()
                    spaces = '  ' * indent
                    for note in sorted(notes, key=lambda note: note.subject()):
                        bf.write('%s%s\n%s%s\n' % (spaces, note.subject(), spaces, note.description()))
                        bf.write(renderNotes(note.children(), indent + 1))
                    return bf.getvalue()
                row.append(renderNotes(item.notes()))
            elif column.name() == 'attachments':
                row.append(u'\n'.join(sorted([attachment.subject() for attachment in item.attachments()])))
            else:
                row.append(column.render(item, humanReadable=False))
        row[0] = self.indent(item) + row[0]
        return row

    def shouldSplitDateAndTime(self, column):
        return self.__separateDateAndTimeColumns and column.name() in self.dateAndTimeColumnHeaders
    
    def splitDateAndTime(self, column, item):
        if column.name() == 'period':
            return self.__splitDateAndTime(item.getStart()) + self.__splitDateAndTime(item.getStop())
        return self.__splitDateAndTime(getattr(item, column.name())())

    def __splitDateAndTime(self, dateTime):
        if dateTime == date.DateTime() or dateTime is None:
            return '', ''
        return render.date(dateTime), render.time(dateTime)

    def itemRows(self, items):
        return [self.itemRow(item) for item in items]
    
    def rows(self, items):
        return [self.headerRow()] + self.itemRows(items)
    

def viewer2csv(viewer, selectionOnly=False, separateDateAndTimeColumns=False,
               columns=None):
    ''' Convert the items displayed by a viewer into a list of rows, where
        each row consists of a list of values. If the viewer is in tree mode, 
        indent the first value (typically the subject of the item) to 
        indicate the depth of the item in the tree. '''
    
    isTree = viewer.isTreeViewer()    
    columns = columns or viewer.visibleColumns()
    rowBuilder = RowBuilder(columns, isTree, separateDateAndTimeColumns)
    items = viewer.visibleItems()
    if selectionOnly:
        items = [item for item in items if viewer.isselected(item)]
        if isTree:
            items = extendedWithAncestors(items)
    return rowBuilder.rows(items)




'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
import test
from taskcoachlib import gui, config



class PrinterTest(test.TestCase):
    def setUp(self):
        super(PrinterTest, self).setUp()
        self.settings = config.Settings(load=False)
        self.margins = dict(top=1, left=2, bottom=3, right=4)
        self.printerSettings = gui.printer.PrinterSettings(self.settings)
        self.pageSetupData = wx.PageSetupDialogData()

    def tearDown(self):
        super(PrinterTest, self).tearDown()
        self.resetPrinterSettings()
        
    def resetPrinterSettings(self):
        gui.printer.PrinterSettings.deleteInstance() # pylint: disable=E1101
 
    def testInitialSettings(self):
        printerSettings = self.printerSettings
        self.assertEqual(wx.Point(0, 0), printerSettings.GetMarginTopLeft())
        self.assertEqual(0, printerSettings.GetPaperId())
        self.assertEqual(wx.PORTRAIT, printerSettings.GetOrientation())

    def testSetMargin(self):
        self.pageSetupData.SetMarginTopLeft(wx.Point(10, 1))
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        self.assertEqual(wx.Point(10, 1), 
                         self.printerSettings.GetMarginTopLeft())

    def testDefaultMarginsFromSettings(self):
        settings = self.settings
        for margin in self.margins:
            self.assertEqual(0, settings.getint('printer', 'margin_'+margin))

    def testSetPaperId(self):
        self.pageSetupData.SetPaperId(1)
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        self.assertEqual(1, self.printerSettings.GetPaperId())

    def testDefaultPaperIdFromSettings(self):
        self.assertEqual(0, self.settings.getint('printer', 'paper_id'))

    def testSetOrientation(self):
        self.pageSetupData.GetPrintData().SetOrientation(wx.LANDSCAPE)
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        self.assertEqual(wx.LANDSCAPE, self.printerSettings.GetOrientation())

    def testDefaultOrientationFromSettings(self):
        self.assertEqual(wx.PORTRAIT, 
                         self.settings.getint('printer', 'orientation'))

    def testUpdateMarginsInPageSetupDataUpdatesSettings(self):
        self.pageSetupData.SetMarginTopLeft(wx.Point(self.margins['left'],
                                                     self.margins['top']))
        self.pageSetupData.SetMarginBottomRight(wx.Point(self.margins['right'], 
                                                         self.margins['bottom']))
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        for margin in self.margins:
            self.assertEqual(self.margins[margin], 
                             self.settings.getint('printer', 'margin_'+margin))

    def testUpdatePaperIdInPageSetupDataUpdatesSettings(self):
        self.pageSetupData.SetPaperId(1)
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        self.assertEqual(1, self.settings.getint('printer', 'paper_id'))

    def testUpdateOrientationInPageSetupDataUpdatesSettings(self):
        self.pageSetupData.GetPrintData().SetOrientation(wx.LANDSCAPE)
        self.printerSettings.updatePageSetupData(self.pageSetupData)
        self.assertEqual(wx.LANDSCAPE, 
                         self.settings.getint('printer', 'orientation'))

    def testMarginsInPageSetupDataAreUpdatedFromSettings(self):
        self.resetPrinterSettings()
        for margin in self.margins:
            self.settings.set('printer', 'margin_'+margin, 
                              str(self.margins[margin]))
        printerSettings = gui.printer.PrinterSettings(self.settings)
        self.assertEqual(wx.Point(2, 1), printerSettings.GetMarginTopLeft())
        self.assertEqual(wx.Point(4, 3), printerSettings.GetMarginBottomRight())
        
    def testPaperIdInPageSetupDataIsUpdatedFromSettings(self):
        self.resetPrinterSettings()
        self.settings.set('printer', 'paper_id', '1')
        printerSettings = gui.printer.PrinterSettings(self.settings)
        self.assertEqual(1, printerSettings.GetPaperId())

    def testOrientationInPageSetupDataIsUpdatedFromSettings(self):
        self.resetPrinterSettings()
        self.settings.set('printer', 'orientation', str(wx.LANDSCAPE))
        printerSettings = gui.printer.PrinterSettings(self.settings)
        self.assertEqual(wx.LANDSCAPE, printerSettings.GetOrientation())


class HTMLPrintoutTest(test.TestCase):
    def testCreate(self):
        gui.printer.HTMLPrintout('<html></html>', config.Settings(load=False))
        


'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2008 Thomas Sonne Olesen <tpo@sonnet.dk>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import render
from taskcoachlib.domain import date
from taskcoachlib.i18n import _
from taskcoachlib.thirdparty.pubsub import pub
import base


class BaseCompositeEffort(base.BaseEffort):  # pylint: disable=W0223        
    def parent(self):
        # Composite efforts don't have a parent.
        return None
        
    def _inPeriod(self, effort):
        return self.getStart() <= effort.getStart() <= self.getStop()
    
    def mayContain(self, effort):
        ''' Return whether effort would be contained in this composite effort 
            if it existed. '''
        return self._inPeriod(effort)

    def __contains__(self, effort):
        return effort in self._getEfforts()

    def __getitem__(self, index):
        return self._getEfforts()[index]

    def __len__(self):
        return len(self._getEfforts())
    
    def _getEfforts(self):
        raise NotImplementedError

    def markDirty(self):
        pass  # CompositeEfforts cannot be dirty
    
    def __doRound(self, duration, rounding, roundUp):
        if rounding:
            return duration.round(seconds=rounding, alwaysUp=roundUp)
        return duration

    def duration(self, recursive=False, rounding=0, roundUp=False):
        return sum((self.__doRound(effort.duration(), rounding, roundUp) for effort in \
                    self._getEfforts(recursive)), date.TimeDelta())

    def isBeingTracked(self, recursive=False):  # pylint: disable=W0613
        return any(effort.isBeingTracked() for effort in self._getEfforts())

    def durationDay(self, dayOffset, rounding=0, roundUp=False):
        ''' Return the duration of this composite effort on a specific day. '''
        startOfDay = self.getStart() + date.TimeDelta(days=dayOffset)
        endOfDay = self.getStart() + date.TimeDelta(days=dayOffset + 1)
        return sum((self.__doRound(effort.duration(), rounding, roundUp) for effort in \
                    self._getEfforts(recursive=False) \
                    if startOfDay <= effort.getStart() <= endOfDay), 
                   date.TimeDelta())
                              
    def notifyObserversOfDurationOrEmpty(self):
        if self._getEfforts():
            self.sendDurationChangedMessage()
        else:
            pub.sendMessage(self.compositeEmptyEventType(), sender=self)
        
    @classmethod
    def compositeEmptyEventType(class_):
        return 'pubsub.effort.composite.empty'
        
    @classmethod
    def modificationEventTypes(class_):
        return []  # A composite effort cannot be 'dirty' since its contents
        # are determined by the contained efforts.

    def onTimeSpentChanged(self, newValue, sender):  # pylint: disable=W0613
        if self._refreshCache():
            # Only need to notify if our time spent actually changed
            self.notifyObserversOfDurationOrEmpty()

    def onRevenueChanged(self, newValue, sender):  # pylint: disable=W0613
        self.sendRevenueChangedMessage()
           
    def revenue(self, recursive=False):
        raise NotImplementedError  # pragma: no cover
    
    def _invalidateCache(self):
        ''' Empty the cache so that it will be filled when accessed. '''
        raise NotImplementedError  # pragma: no cover
    
    def _refreshCache(self):
        ''' Refresh the cache right away and return whether the cache was 
            actually changed. '''
        raise NotImplementedError  # pragma: no cover
    

class CompositeEffort(BaseCompositeEffort):
    ''' CompositeEffort is a lazy list (but cached) of efforts for one task
        (and its children) and within a certain time period. The task, start 
        of time period and end of time period need to be provided when
        initializing the CompositeEffort and cannot be changed
        afterwards. '''
    
    def __init__(self, task, start, stop):  # pylint: disable=W0621
        super(CompositeEffort, self).__init__(task, start, stop)
        self.__hash_value = hash((task, start))
        # Effort cache: {True: [efforts recursively], False: [efforts]}
        self.__effort_cache = dict()  
        '''
        FIMXE! CompositeEffort does not derive from base.Object
        patterns.Publisher().registerObserver(self.onAppearanceChanged,
            eventType=task.appearanceChangedEventType(), eventSource=task)
        '''

    def __hash__(self):
        return self.__hash_value

    def __repr__(self):
        return 'CompositeEffort(task=%s, start=%s, stop=%s, efforts=%s)' % \
            (self.task(), self.getStart(), self.getStop(),
            str([e for e in self._getEfforts()]))

    def addEffort(self, anEffort):
        assert self._inPeriod(anEffort)
        self.__effort_cache.setdefault(True, set()).add(anEffort)
        if anEffort.task() == self.task():
            self.__effort_cache.setdefault(False, set()).add(anEffort)
            
    def revenue(self, recursive=False):
        return sum(effort.revenue() for effort in self._getEfforts(recursive))
    
    def _invalidateCache(self):
        self.__effort_cache = dict()
        
    def _refreshCache(self, recursive=None):
        recursive_values = (False, True) if recursive is None else (recursive,)
        previous_cache = self.__effort_cache.copy()
        cache_changed = False
        for recursive in recursive_values:
            cache = self.__effort_cache[recursive] = \
                set([effort for effort in \
                     self.task().efforts(recursive=recursive) if \
                     self._inPeriod(effort)])
            if cache != previous_cache.get(recursive, set()):
                cache_changed = True
        return cache_changed
                
    def _getEfforts(self, recursive=True):  # pylint: disable=W0221
        if recursive not in self.__effort_cache:
            self._refreshCache(recursive=recursive)
        return list(self.__effort_cache[recursive])
    
    def mayContain(self, effort):
        ''' Return whether effort would be contained in this composite effort 
            if it existed. '''
        return effort.task() == self.task() and \
            super(CompositeEffort, self).mayContain(effort)
            
    def description(self):
        if len(set(effort.description() for effort in self._getEfforts(False))) == 1:
            return self._getEfforts(False)[0].description()
        effortDescriptions = [effort.description() for effort in \
                              sorted(self._getEfforts(False), 
                                     key=lambda effort: effort.getStart()) if effort.description()]
        return '\n'.join(effortDescriptions)
    
    def onAppearanceChanged(self, event):    
        return  # FIXME: CompositeEffort does not derive from base.Object
        #patterns.Event(self.appearanceChangedEventType(), self, event.value()).send()


class CompositeEffortPerPeriod(BaseCompositeEffort):
    class Total(object):
        # pylint: disable=W0613
        def subject(self, *args, **kwargs): 
            return _('Total')
        
        def foregroundColor(self, *args, **kwargs):
            return None
        
        def backgroundColor(self, *args, **kwargs):
            return None
        
        def font(self, *args, **kwargs):
            return None
        
    total = Total()
        
    def __init__(self, start, stop, taskList, initialEffort=None):
        self.taskList = taskList
        super(CompositeEffortPerPeriod, self).__init__(None, start, stop)
        if initialEffort:
            assert self._inPeriod(initialEffort)
            self.__effort_cache = [initialEffort]
        else:
            self._invalidateCache()
            
    def addEffort(self, anEffort):
        assert self._inPeriod(anEffort)
        if anEffort not in self.__effort_cache:
            self.__effort_cache.append(anEffort)

    @classmethod
    def task(cls):
        return cls.total

    def isTotal(self):
        return True

    def description(self, *args, **kwargs):  # pylint: disable=W0613
        return _('Total for %s') % render.dateTimePeriod(self.getStart(), 
                                                         self.getStop())

    def revenue(self, recursive=False):  # pylint: disable=W0613
        return sum(effort.revenue() for effort in self._getEfforts())

    def categories(self, *args, **kwargs):
        return [] 
    
    def tasks(self):
        ''' Return the tasks that have effort in this period. '''
        return set([effort.task() for effort in self._getEfforts()])
        
    def __repr__(self):
        return 'CompositeEffortPerPeriod(start=%s, stop=%s, efforts=%s)' % \
            (self.getStart(), self.getStop(),
            str([e for e in self._getEfforts()]))
            
    # Cache handling:

    def _getEfforts(self, recursive=False):  # pylint: disable=W0613,W0221
        if self.__effort_cache is None:
            self._refreshCache()
        return self.__effort_cache
    
    def _invalidateCache(self):
        self.__effort_cache = None

    def _refreshCache(self):
        previous_cache = [] if self.__effort_cache is None else self.__effort_cache[:]
        self.__effort_cache = []
        self.__add_task_effort_to_cache(self.taskList)
        return previous_cache != self.__effort_cache

    def __add_task_effort_to_cache(self, tasks):
        ''' Add the effort of the tasks to the cache. '''
        for task in tasks:
            effort_in_period = [effort for effort in task.efforts() if \
                                self._inPeriod(effort)]
            self.__effort_cache.extend(effort_in_period)



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx


class FontPickerCtrl(wx.Button):
    def __init__(self, *args, **kwargs):
        self.__font = kwargs.pop('font')
        self.__colour = kwargs.pop('colour')
        super(FontPickerCtrl, self).__init__(*args, **kwargs)
        self.__updateButton()
        self.Bind(wx.EVT_BUTTON, self.onClick)

    def GetSelectedFont(self):
        return self.__font
    
    def SetSelectedFont(self, font):
        self.__font = font
        self.__updateButton()

    def GetSelectedColour(self):
        return self.__colour

    def SetSelectedColour(self, colour):
        self.__colour = colour
        self.__updateButton()

    def onClick(self, event):
        event.Skip(False)
        dialog = wx.FontDialog(self, self.__newFontData())
        if wx.ID_OK == dialog.ShowModal():
            self.__readFontData(dialog.GetFontData())
            self.__updateButton()
            self.__sendPickerEvent()

    def __newFontData(self):
        fontData = wx.FontData()
        fontData.SetInitialFont(self.__font)
        fontData.SetColour(self.__colour)
        return fontData

    def __readFontData(self, fontData):
        self.__font = fontData.GetChosenFont()
        self.__colour = fontData.GetColour()
    
    def __updateButton(self):
        self.SetLabel(self.__font.GetNativeFontInfoUserDesc())
        self.SetFont(self.__font)
        self.SetForegroundColour(self.__colour)

    def __sendPickerEvent(self):
        event = wx.FontPickerEvent(self, self.GetId(), self.__font)
        self.GetEventHandler().ProcessEvent(event)



# -*- coding: UTF-8 -*-
#This is generated code - do not edit
encoding = 'UTF-8'
dict = {'Insert a new attachment': 'Neue Datei hinzuf\xc3\xbcgen', 'Closed %s': '%s geschlossen', 'Monthly': 'Monatlich', 'Minimize priority': 'Niedrigste Priorit\xc3\xa4t', 'Access': 'Zugriff', 'New subnote': 'Neue Unternotiz', '&New task...': '&Neue Aufgabe...', 'Language not found?': 'Sprache nicht gefunden?', 'Task': 'Aufgabe', 'Notes': 'Notizen', 'Decrease priority': 'Priorit\xc3\xa4t verringern', 'Add attachment': 'Anhang hinzuf\xc3\xbcgen', 'Yearly': 'J\xc3\xa4hrlich', '%s file error': '%s Dateifehler', 'Edit location of attachments': 'Bearbeite den Ort des Anhangs', 'Copy "%s"': 'Kopiere "%s"', 'Weekly': 'W\xc3\xb6chentlich', 'New task': 'Neue Aufgabe', 'Browse': 'Durchsuchen', 'Error while saving %s.ini:\n%s\n': 'Fehler beim Speichern von %s.ini:\n%s\n', 'Dates': 'Daten', 'Priority': 'Priorit\xc3\xa4t', 'Location': 'Ort', 'Show tips window on startup': 'Hilfefenster beim Start anzeigen', 'keeping dates on the same weekday': 'gleichen Wochentag beibehalten', 'New subtask': 'Neue Unteraufgabe', 'Insert a new category': 'Neue Kategorie einf\xc3\xbcgen', 'Enable SyncML': 'SyncML aktivieren', 'User name/ID': 'Benutzername/ID', 'Always': 'Immer', 'Categories': 'Kategorien', 'New &subtask...': 'Neue &Teilaufgabe...', 'None': 'Keine', 'Edit attachment "%s" location': 'Bearbeite Speicherort des Anhangs "%s"', 'Budget': 'Budget', 'Daily': 'T\xc3\xa4glich', 'Delete': 'Entfernen', 'Tasks': 'Aufgaben', 'Completion date': 'Abschlussdatum', 'Due date': 'F\xc3\xa4lligkeitsdatum', 'Maximize priority': 'H\xc3\xb6chste Priorit\xc3\xa4t', 'day(s),': 'Tag(e),', 'Yes': 'Ja', 'Subject': 'Betreff', 'Cut': 'Ausschneiden', 'No': 'Nein', 'Start': 'Start', 'Reminder': 'Erinnerung', 'Check for new version of %(name)s on startup': 'Beim Start auf neue Version von %(name)s pr\xc3\xbcfen', 'Start tracking': 'Stoppuhr starten', 'Open the selected attachments': 'Ausgew\xc3\xa4hlte Anh\xc3\xa4ngen \xc3\xb6ffnen', 'Drag and drop': 'Ziehen und Ablegen', 'New attachment': 'Neuer Anhang', 'Stop tracking': 'Stoppuhr anhalten', 'Language': 'Sprache', 'Save error': 'Fehler beim Speichern', 'Insert a new note': 'Neue Notiz einf\xc3\xbcgen', 'Error': 'Failer', 'Edit task': 'Aufgabe editieren', 'Attachments': 'Anh\xc3\xa4nge', 'month(s),': 'Monat(e),', 'Window behavior': 'Fensterverhalten', 'Insert a new task': 'Neue Aufgabe einf\xc3\xbcgen', 'Auto save after every change': 'Nach jeder \xc3\x84nderung speichern', 'Error while reading %s:\n': 'Fehler beim Lesen %s\n', 'Open': '\xc3\x96ffnen', 'Editor': 'Editor', 'Slow': 'Langsam', 'Hourly fee': 'Stundensatz', 'New subcategory': 'Neue Unterkategorie', 'Recurrence': 'Wiederholung', 'Increase priority': 'Priorit\xc3\xa4t erh\xc3\xb6hen', 'New note': 'Neue Notiz', 'New version of %(name)s available': 'Neue Version von %(name)s verf\xc3\xbcgbar', 'Description': 'Beschreibung', 'Never': 'Nie', 'Stop': 'Stop', 'Copy': 'Kopieren', 'Paste': 'Einf\xc3\xbcgen', ', every': ', jeden', 'Errors have occured. Please see "%s"': 'Es sind Fehler aufgetreten. Weitere Informationen siehe "%s"', 'period,': 'Zeitspanne', 'Add an effort period to the selected task(s)': 'Aufwand auf das Budget der ausgew\xc3\xa4hlten Aufgabe(n) buchen', 'New attachment...': 'Neuer Anhang...', "Couldn't load settings from TaskCoach.ini:\n%s": 'Konnte Einstellungen von TaskCoach.ini nicht laden:\n%s', 'Add attachment to "%s"': 'Anhang hinzuf\xc3\xbcgen zu "%s"', 'Errors have occured. Please see "taskcoachlog.txt" in your "My Documents" folder.': 'Es sind Fehler aufgetreten. Weitere Informationen wurde in der Datei "taskcoachlog.txt" im Ordner "Eigene Dateien" gespeichert.', 'Open attachment': 'Anhang \xc3\xb6ffnen', 'New category': 'Neue Kategorie', 'year(s),': 'Jahr(e)', 'Remove attachment': 'Anhang entfernen', 'week(s),': 'Woche(n),'}


'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from taskcoachlib import meta, patterns, command, render, operating_system, \
    speak
from taskcoachlib.domain import date
from taskcoachlib.i18n import _
from taskcoachlib.thirdparty.pubsub import pub
from wx.lib import sized_controls
import subprocess
import wx


class ReminderDialog(patterns.Observer, sized_controls.SizedDialog):
    def __init__(self, task, taskList, effortList, settings, *args, **kwargs):
        kwargs['title'] = _('%(name)s reminder - %(task)s') % \
            dict(name=meta.name, task=task.subject(recursive=True))
        super(ReminderDialog, self).__init__(*args, **kwargs)
        self.SetIcon(wx.ArtProvider_GetIcon('taskcoach', wx.ART_FRAME_ICON, 
                                            (16, 16)))
        self.task = task
        self.taskList = taskList
        self.effortList = effortList
        self.settings = settings
        self.registerObserver(self.onTaskRemoved, 
                              eventType=self.taskList.removeItemEventType(),
                              eventSource=self.taskList)
        pub.subscribe(self.onTaskCompletionDateChanged, 
                      task.completionDateTimeChangedEventType())
        pub.subscribe(self.onTrackingChanged, task.trackingChangedEventType())
        self.openTaskAfterClose = self.ignoreSnoozeOption = False
        pane = self.GetContentsPane()
        pane.SetSizerType("form")
        
        wx.StaticText(pane, label=_('Task') + ':')
        panel = wx.Panel(pane)
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        self.openTask = wx.Button(panel, 
                                  label=self.task.subject(recursive=True))
        self.openTask.Bind(wx.EVT_BUTTON, self.onOpenTask)
        sizer.Add(self.openTask, flag=wx.ALIGN_CENTER_VERTICAL)
        self.startTracking = wx.BitmapButton(panel)
        self.setTrackingIcon()
        self.startTracking.Bind(wx.EVT_BUTTON, self.onStartOrStopTracking)
        sizer.Add((3, -1), flag=wx.ALIGN_CENTER_VERTICAL)
        sizer.Add(self.startTracking, flag=wx.ALIGN_CENTER_VERTICAL)
        panel.SetSizerAndFit(sizer)
        
        for label in _('Reminder date/time') + ':', \
            render.dateTime(self.task.reminder()), _('Snooze') + ':':
            wx.StaticText(pane, label=label)
            
        self.snoozeOptions = wx.ComboBox(pane, style=wx.CB_READONLY)
        sizer.Add(self.snoozeOptions, flag=wx.ALIGN_CENTER_VERTICAL)
        snoozeTimesUserWantsToSee = [0] + self.settings.getlist('view', 
                                                                'snoozetimes')
        defaultSnoozeTime = self.settings.getint('view', 'defaultsnoozetime')
        # Use the 1st non-zero option if we don't find the last snooze time:
        selectionIndex = 1  
        # pylint: disable=E1101
        for minutes, label in date.snoozeChoices:
            if minutes in snoozeTimesUserWantsToSee:
                self.snoozeOptions.Append(label, 
                                          date.TimeDelta(minutes=minutes))
                if minutes == defaultSnoozeTime:
                    selectionIndex = self.snoozeOptions.Count - 1
        self.snoozeOptions.SetSelection(min(selectionIndex, 
                                            self.snoozeOptions.Count - 1))
        
        wx.StaticText(pane, label='')
        self.replaceDefaultSnoozeTime = wx.CheckBox(pane, 
            label=_('Also make this the default snooze time for future '
                    'reminders'))
        self.replaceDefaultSnoozeTime.SetValue(self.settings.getboolean('view', 
                                               'replacedefaultsnoozetime'))
        
        buttonSizer = self.CreateStdDialogButtonSizer(wx.OK)
        self.markCompleted = wx.Button(self, label=_('Mark task completed'))
        self.markCompleted.Bind(wx.EVT_BUTTON, self.onMarkTaskCompleted)
        if self.task.completed():
            self.markCompleted.Disable()
        buttonSizer.Add(self.markCompleted, flag=wx.ALIGN_CENTER_VERTICAL)
        self.SetButtonSizer(buttonSizer)
        self.Bind(wx.EVT_CLOSE, self.onClose)
        self.Bind(wx.EVT_BUTTON, self.onOK, id=self.GetAffirmativeId())
        self.Fit()
        self.RequestUserAttention()
        if self.settings.getboolean('feature', 'sayreminder'):
            speak.Speaker().say('"%s: %s"' % (_('Reminder'), task.subject()))

    def onOpenTask(self, event):  # pylint: disable=W0613
        self.openTaskAfterClose = True
        self.Close()
        
    def onStartOrStopTracking(self, event):  # pylint: disable=W0613
        if self.task.isBeingTracked():
            command.StopEffortCommand(self.effortList).do()
        else:
            command.StartEffortCommand(self.taskList, [self.task]).do()
        self.setTrackingIcon()
        
    def onTrackingChanged(self, newValue, sender):  # pylint: disable=W0613
        self.setTrackingIcon()
        
    def setTrackingIcon(self):
        icon = 'clock_stop_icon' if self.task.isBeingTracked() else 'clock_icon'
        self.startTracking.SetBitmapLabel(wx.ArtProvider_GetBitmap(icon, 
            wx.ART_TOOLBAR, (16, 16)))
        
    def onMarkTaskCompleted(self, event):  # pylint: disable=W0613
        self.ignoreSnoozeOption = True
        self.Close()
        command.MarkCompletedCommand(self.taskList, [self.task]).do()
    
    def onTaskRemoved(self, event):
        if self.task in event.values():
            self.Close()
            
    def onTaskCompletionDateChanged(self, newValue, sender):  # pylint: disable=W0613
        if sender == self.task:
            if self.task.completed():
                self.Close()
            else:
                self.markCompleted.Enable()
    
    def onClose(self, event):
        event.Skip()
        replace_default_snooze_time = self.replaceDefaultSnoozeTime.GetValue()
        if replace_default_snooze_time:
            # pylint: disable=E1101
            selection = self.snoozeOptions.Selection
            minutes = self.snoozeOptions.GetClientData(selection).minutes()
            self.settings.set('view', 'defaultsnoozetime', str(int(minutes)))
        self.settings.setboolean('view', 'replacedefaultsnoozetime', 
                                 replace_default_snooze_time)
        self.removeInstance()
        
    def onOK(self, event):
        event.Skip()
        self.Close()



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>
Copyright (C) 2012 Nicola Chiapolini <nicola.chiapolini@physik.uzh.ch>
Copyright (C) 2008 Rob McMullen <rob.mcmullen@gmail.com>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import wx
from taskcoachlib import meta


defaults = {
'balloontips': {
    'customizabletoolbars': 'True',
    'customizabletoolbars_dnd': 'True',
    'filtershiftclick': 'True',
    'autosavehint': 'True',
    'manualordering': 'True',
    'treemanualordering': 'True',
    'treechildrenmanualordering': 'True',
    },
'view': {
    'statusbar': 'True',
    'toolbar': '(22, 22)',
    'toolbarperspective': 'FileOpen,Print,Separator,EditUndo,EditRedo,Separator,EffortStartButton,EffortStop',
    # Index of the active effort viewer in task editor:
    'effortviewerintaskeditor': '0',  
    'taskviewercount': '1',  # Number of task viewers in main window
    'categoryviewercount': '1',  # Number of category viewers in main window
    'noteviewercount': '0',  # Number of note viewers in main window
    'effortviewercount': '0',  # Number of effort viewers in main window
    'effortviewerforselectedtaskscount': '0',
    'squaretaskviewercount': '0',
    'timelineviewercount': '0',
    'calendarviewercount': '0',
    'hierarchicalcalendarviewercount': '0',
    'taskstatsviewercount': '0',
    # Language and locale, maybe set externally (e.g. by PortableApps):
    'language': '',  
    # Language and locale as set by user via preferences, overrides language:
    'language_set_by_user': '',  
    'categoryfiltermatchall': 'False',
    'weekstart': 'monday',  # Start of work week, 'monday' or 'sunday'
    # The next three options are used in the effort dialog to populate the
    # drop down menu with start and stop times.
    'efforthourstart': '8',  # Earliest time, i.e. start of working day
    'efforthourend': '18',  # Last time, i.e. end of working day
    'effortminuteinterval': '15',  # Generate times with this interval
    'snoozetimes': "[5, 10, 15, 30, 60, 120, 1440]",
    'defaultsnoozetime': '5',  # Default snooze time
    'replacedefaultsnoozetime': 'True',  # Make chosen snooze time the default?
    'perspective': '',  # The layout of the viewers in the main window
    # What to do when changing the planned start date or due date:
    'datestied': '',  
    # Default date and times to offer in the task dialog, see preferences for
    # possible values:
    'defaultplannedstartdatetime': 'propose_today_currenttime',
    'defaultduedatetime': 'propose_tomorrow_endofworkingday',
    'defaultactualstartdatetime': 'propose_today_currenttime',
    'defaultcompletiondatetime': 'propose_today_currenttime',
    'defaultreminderdatetime': 'propose_tomorrow_startofworkingday',
    # Show messages from the developers downloaded from the website:
    'developermessages': 'True',
    'lastdevelopermessage': '',
    'descriptionpopups': 'True',
    },
'taskviewer': {
    'title': '',  # User supplied viewer title
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Separator,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,TaskViewerTreeOrListChoice,Search',
    'treemode': 'True',  # True = tree mode, False = list mode
    'sortby': '["dueDateTime"]',
    'sortbystatusfirst': 'True',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "['plannedStartDateTime', 'dueDateTime']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'notes': 28, 'ordering': 28}",
    'columnautoresizing': 'True',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideactivetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False',
    },              
'taskstatsviewer': {
    'title': '',
    'toolbarperspective': 'TaskNew,TaskNewFromTemplateButton,Separator,ViewerPieChartAngle,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideactivetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False',
    'piechartangle': '30',
    },
'prerequisiteviewerintaskeditor': {
    'title': '',  # User supplied viewer title
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'treemode': 'True',  # True = tree mode, False = list mode
    'sortby': '["subject"]',
    'sortbystatusfirst': 'True',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "['prerequisites', 'dependencies', 'plannedStartDateTime', "
               "'dueDateTime']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'notes': 28}",
    'columnautoresizing': 'True',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hideactivetasks': 'False',
    'hidecompositetasks': 'False'
    },
'squaretaskviewer': {
    'title': '',
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Separator,SquareTaskViewerOrderChoice,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'sortby': '["budget"]',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideactivetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False'
    },
'timelineviewer': {
    'title': '',
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hideactivetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False'
    },
'hierarchicalcalendarviewer': {
    'title': '',
    'calendarformat': '0',
    'headerformat': '1',
    'drawnow': 'True',
    'todaycolor': '0, 0, 200',
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Separator,HierarchicalCalendarViewerConfigure,HierarchicalCalendarViewerPreviousPeriod,HierarchicalCalendarViewerToday,HierarchicalCalendarViewerNextPeriod,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideactivetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'sortbystatusfirst': 'True',
    },
'calendarviewer': {
    'title': '',
    'toolbarperspective': 'TaskNew,NewSubItem,TaskNewFromTemplateButton,Separator,Edit,Delete,Separator,TaskMarkInactive,TaskMarkActive,TaskMarkCompleted,Separator,EffortStart,EffortStop,Separator,CalendarViewerConfigure,CalendarViewerPreviousPeriod,CalendarViewerToday,CalendarViewerNextPeriod,Spacer,ViewerHideTasks_completed,ViewerHideTasks_inactive,ResetFilter,Search',
    'viewtype': '1',
    'periodcount': '1',
    'periodwidth': '150',
    'vieworientation': '1',
    'viewdate': '',
    'gradient': 'False',
    'shownostart': 'False',
    'shownodue': 'False',
    'showunplanned': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'hideinactivetasks': 'False',
    'hidelatetasks': 'False',
    'hideactivetasks': 'False',
    'hideduesoontasks': 'False',
    'hideoverduetasks': 'False',
    'hidecompletedtasks': 'False',
    'hidecompositetasks': 'False',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'sortbystatusfirst': 'True',
    'highlightcolor': '',
    'shownow': 'True' 
    },
'categoryviewer': {
    'title': '',
    'toolbarperspective': 'CategoryNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'notes': 28, 'ordering': 28}",
    'columnautoresizing': 'True'
    },
'categoryviewerintaskeditor': {
    'title': '',
    'toolbarperspective': 'CategoryNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'notes': 28, 'ordering': 28}",
    'columnautoresizing': 'True'
    },
'categoryviewerinnoteeditor': {
    'title': '',
    'toolbarperspective': 'CategoryNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'notes': 28, 'ordering': 28}",
    'columnautoresizing': 'True'
    },
'noteviewer': {
    'title': '',
    'toolbarperspective': 'NoteNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "['attachments', 'description', 'creationDateTime', \
                 'modificationDateTime']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'description': 200, 'ordering': 28}",
    'columnautoresizing': 'True'
    },
'noteviewerintaskeditor': {
    'toolbarperspective': 'NoteNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'columns': "['attachments', 'description', 'creationDateTime', \
                 'modificationDateTime']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'attachments': 28, 'description': 200, 'ordering': 28}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    },
'noteviewerincategoryeditor': {
    'toolbarperspective': 'NoteNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'columns': "['subject']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'ordering': 28}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    },
'noteviewerinattachmenteditor': {
    'toolbarperspective': 'NoteNew,NewSubItem,Separator,Edit,Delete,Spacer,ResetFilter,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'columns': "['subject']",
    'columnsalwaysvisible': "['subject']",
    'columnwidths': "{'ordering': 28}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    },
'effortviewer': {
    'title': '',
    'toolbarperspective': 'EffortNew,Separator,Edit,Delete,Separator,EffortStartForEffort,EffortStop,Separator,EffortViewerAggregationChoice,Spacer,ResetFilter,Search',
    'aggregation': 'details',  # 'details' (default), 'day', 'week', or 'month'
    'sortby': '["-period"]',
    'sortcasesensitive': 'False',
    'columns': "['description', 'timeSpent']",
    'columnsalwaysvisible': "['period', 'task']",
    'columnwidths': "{'period': 160, 'monday': 70, 'tuesday': 70, "
                     "'wednesday': 70, 'thursday': 70, 'friday': 70, "
                     "'saturday': 70, 'sunday': 70, 'description': 200}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'round': '0',  # round effort to this number of seconds, 0 = no rounding
    'alwaysroundup': 'False',
    },
'effortviewerforselectedtasks': {
    'title': '',
    'toolbarperspective': 'EffortNew,Separator,Edit,Delete,Separator,EffortStartForEffort,EffortStop,Separator,EffortViewerAggregationChoice,Spacer,ResetFilter,Search',
    'aggregation': 'details',  # 'details' (default), 'day', 'week', or 'month'
    'sortby': '["-period"]',
    'sortcasesensitive': 'False',
    'columns': "['description', 'timeSpent']",
    'columnsalwaysvisible': "['period', 'task']",
    'columnwidths': "{'period': 160, 'monday': 70, 'tuesday': 70, "
                     "'wednesday': 70, 'thursday': 70, 'friday': 70, "
                     "'saturday': 70, 'sunday': 70, 'description': 200}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'round': '0',  # round effort to this number of seconds, 0 = no rounding
    'alwaysroundup': 'False',
    },
'effortviewerintaskeditor': {
    'toolbarperspective': 'EffortNew,Separator,Edit,Delete,Separator,EffortStartForEffort,EffortStop,Separator,EffortViewerAggregationChoice,Spacer,ResetFilter,Search',
    'aggregation': 'details',  # 'details' (default), 'day', 'week', or 'month'
    'sortby': '["-period"]',
    'sortcasesensitive': 'False',
    'columns': "['description', 'timeSpent']",
    'columnsalwaysvisible': "['period', 'task']",
    'columnwidths': "{'period': 160, 'monday': 70, 'tuesday': 70, "
                     "'wednesday': 70, 'thursday': 70, 'friday': 70, "
                     "'saturday': 70, 'sunday': 70, 'description': 200}",
    'columnautoresizing': 'True',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'round': '0',  # round effort to this number of seconds, 0 = no rounding
    'alwaysroundup': 'False', 
    },
'attachmentviewer': {
    'title': '',
    'toolbarperspective': 'AttachmentNew,Separator,Edit,Delete,Separator,AttachmentOpen,Spacer,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['type', 'subject']",
    'columnwidths': "{'notes': 28, 'type': 28}",
    'columnautoresizing': 'True'
    },
'attachmentviewerintaskeditor': {
    'title': '',
    'toolbarperspective': 'AttachmentNew,Separator,Edit,Delete,Separator,AttachmentOpen,Spacer,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['type', 'subject']",
    'columnwidths': "{'notes': 28, 'type': 28}",
    'columnautoresizing': 'True'
    },
'attachmentviewerinnoteeditor': {
    'title': '',
    'toolbarperspective': 'AttachmentNew,Separator,Edit,Delete,Separator,AttachmentOpen,Spacer,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['type', 'subject']",
    'columnwidths': "{'notes': 28, 'type': 28}",
    'columnautoresizing': 'True'
    },
'attachmentviewerincategoryeditor': {
    'title': '',
    'toolbarperspective': 'AttachmentNew,Separator,Edit,Delete,Separator,AttachmentOpen,Spacer,Search',
    'sortby': '["subject"]',
    'sortcasesensitive': 'False',
    'searchfilterstring': '',
    'searchfiltermatchcase': 'False',
    'searchfilterincludesubitems': 'False',
    'searchdescription': 'False',
    'regularexpression': 'False',
    'columns': "[]",
    'columnsalwaysvisible': "['type', 'subject']",
    'columnwidths': "{'notes': 28, 'type': 28}",
    'columnautoresizing': 'True' 
    },
'window': {
    'size': '(900, 500)',  # Default size of the main window
    'position': '(-1, -1)',  # Position of the main window, undefined by default
    'iconized': 'False',  # Don't start up iconized by default
    'maximized': 'False',  # Don't start up maximized by default
    # Possible strticonized values: 'Never', 'Always', 'WhenClosedIconized'
    'starticonized': 'WhenClosedIconized',  
    'splash': 'True',  # Show a splash screen while starting up
    'hidewheniconized': 'False',  # Don't hide the window from the task bar
    'hidewhenclosed': 'False',  # Close window quits the application
    'tips': 'True',  # Show tips after starting up
    'tipsindex': '0',  # Start at the first tip
    'blinktaskbariconwhentrackingeffort': 'True'
    },
'effortdialog': {
    'size': '(-1, -1)',  # Size of the dialogs, calculated by default
    'position': '(-1, -1)',  # Position of the dialog, undefined by default
    'maximized': 'False'  # Don't open the dialog maximized by default
    },
'file': {
    'recentfiles': '[]',
    'maxrecentfiles': '9',
    'lastfile': '',
    'autosave': 'True',
    'autoload': 'False',
    # Formats to automatically import from, only "Todo.txt" supported at this 
    # time:
    'autoimport': '[]',  
    # Formats to automatically export to, only "Todo.txt" supported at this 
    # time:
    'autoexport': '[]',  
    'nopoll': 'False',
    'saveinifileinprogramdir': 'False',
    'attachmentbase': '',
    'lastattachmentpath': '',
    'inifileloaded': 'True',
    'inifileloaderror': ''
    },
'fgcolor': {
    'activetasks': '(0, 0, 0, 255)',
    'latetasks': '(160, 32, 240, 255)',
    'completedtasks': '(0, 255, 0, 255)',
    'overduetasks': '(255, 0, 0, 255)',
    'inactivetasks': '(192, 192, 192, 255)',
    'duesoontasks': '(255, 128, 0, 255)' 
    },
'bgcolor': {
    'activetasks': '(255, 255, 255, 255)',
    'latetasks': '(255, 255, 255, 255)',
    'completedtasks': '(255, 255, 255, 255)',
    'overduetasks': '(255, 255, 255, 255)',
    'inactivetasks': '(255, 255, 255, 255)',
    'duesoontasks': '(255, 255, 255, 255)'
    },
'font': {
    'activetasks': '',
    'latetasks': '',
    'completedtasks': '',
    'overduetasks': '',
    'inactivetasks': '',
    'duesoontasks': ''
    },
'icon': { 
    'activetasks': 'led_blue_icon',
    'latetasks': 'led_purple_icon',
    'completedtasks': 'checkmark_green_icon',
    'overduetasks': 'led_red_icon',
    'inactivetasks': 'led_grey_icon',
    'duesoontasks': 'led_orange_icon' 
    },
'editor': {
    'descriptionfont': '',  # Font to use in the desciption field of editors
    'maccheckspelling': 'True' 
    },
'os_darwin': {
    'getmailsubject': 'False'
    },
'os_linux': {
    'focustextentry': 'True'
    },
'version': {
    'python': '',  # Filled in by the Settings class when saving the settings
    'wxpython': '',  # Idem
    'pythonfrozen': '',  # Idem
    'current': meta.data.version,
    'notified': meta.data.version,
    'notify': 'True' 
    },
'behavior': {
    'markparentcompletedwhenallchildrencompleted': 'False',
    'duesoonhours': '24'  # When a task is considered to be "due soon"
    }, 
'feature': {
    'syncml': 'False',
    'iphone': 'False',
    'notifier': 'Task Coach',
    'minidletime': '0',
    'usesm2': 'False',
    'showsmwarning': 'True',
    'sayreminder': 'False',
    'sdtcspans': '60,120,1440,2880',
    'sdtcspans_effort': '60,120,180,240',
    'decimaltime' : 'False'
    },
'syncml': { 
    'url': '',
    'username': '',
    'preferredsyncmode': 'TWO_WAY',
    'verbose': 'True',
    'taskdbname': 'task',
    'notedbname': 'note',
    'synctasks': 'True',
    'syncnotes': 'True',
    'showwarning': 'True' 
    },
'iphone': {
    'password': '',
    'service': '',
    'synccompleted': 'True',
    'showlog': 'False' 
    },
'printer': {
    'margin_left': '0',
    'margin_top': '0',
    'margin_bottom': '0',
    'margin_right': '0',
    'paper_id': '0',
    'orientation': str(wx.PORTRAIT) 
    },
'export': {
    'html_selectiononly': 'False',
    'html_separatecss': 'False',
    'csv_selectiononly': 'False',
    'csv_separatedateandtimecolumns': 'False',
    'ical_selectiononly': 'False',
    'todotxt_selectiononly': 'False' 
    }
}

minimum = { 
'view': { 
    'taskviewercount': '1' 
    }
}



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test, mock
from taskcoachlib.domain import task, category


class TaskViewerAndCategoryFilterIntegrationTestFixture(test.wxTestCase):
    def setUp(self):
        super(TaskViewerAndCategoryFilterIntegrationTestFixture, self).setUp()
        self.app = mock.App()
        parent = task.Task('parent')
        child = task.Task('child')
        parent.addChild(child)
        self.category = category.Category('category')
        self.app.mainwindow.taskFile.categories().append(self.category)
        self.app.mainwindow.taskFile.tasks().extend([parent, child])
        self.category.addCategorizable(child)
        self.category.setFiltered()
        self.taskViewer = self.app.mainwindow.viewer[0]
        
    def tearDown(self):
        mock.App.deleteInstance()
        super(TaskViewerAndCategoryFilterIntegrationTestFixture, self).tearDown()
        

class TaskListViewerAndCategoryFilterIntegrationTest( \
        TaskViewerAndCategoryFilterIntegrationTestFixture):
            
    def testFilterOnCategoryChildDoesHideParent(self):
        self.taskViewer.settings.setboolean(self.taskViewer.settingsSection(), 'treemode', False)
        self.assertEqual(1, self.taskViewer.widget.GetItemCount())


class TaskTreeViewerAndCategoryFilterIntegrationTest( \
        TaskViewerAndCategoryFilterIntegrationTestFixture):
            
    def testFilterOnCategoryChildDoesNotHideParent(self):
        self.taskViewer.settings.setboolean(self.taskViewer.settingsSection(), 'treemode', True)
        self.taskViewer.expandAll()
        self.assertEqual(2, self.taskViewer.widget.GetItemCount())
        
    



'''
Task Coach - Your friendly task manager
Copyright (C) 2004-2016 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import singleton as patterns


class Command(object):
    def __init__(self, *args, **kwargs):
        super(Command, self).__init__() # object.__init__ takes no arguments
        
    def do(self):
        CommandHistory().append(self)

    def undo(self):
        pass

    def redo(self):
        pass

    def __str__(self):
        return 'command'


class CommandHistory(object):
    __metaclass__ = patterns.Singleton

    def __init__(self):
        self.__history = []
        self.__future = []

    def append(self, command):
        self.__history.append(command)
        del self.__future[:]

    def undo(self):
        if self.__history:
            command = self.__history.pop()
            command.undo()
            self.__future.append(command)

    def redo(self):
        if self.__future:
            command = self.__future.pop()
            command.redo()
            self.__history.append(command)

    def clear(self):
        del self.__history[:]
        del self.__future[:]

    def hasHistory(self):
        return self.__history
        
    def getHistory(self):
        return self.__history
        
    def hasFuture(self):
        return self.__future

    def getFuture(self):
        return self.__future
        
    def _extendLabel(self, label, commandList):
        if commandList:
            commandName = u' %s'%commandList[-1]
            label += commandName.lower()
        return label

    def undostr(self, label='Undo'):
        return self._extendLabel(label, self.__history)

    def redostr(self, label='Redo'):
        return self._extendLabel(label, self.__future)



'''
Task Coach - Your friendly task manager
Copyright (C) 2011 Task Coach developers <developers@taskcoach.org>

Task Coach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Task Coach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

import test, wx
from taskcoachlib.changes import ChangeMonitor
from taskcoachlib.domain.base import Object, CompositeObject
from taskcoachlib.patterns import ObservableList


class MonitorBaseTest(test.TestCase):
    klass = Object
    listClass = ObservableList

    def setUp(self):
        self.monitor = ChangeMonitor()
        self.monitor.monitorClass(self.klass)

        self.list = self.listClass()
        self.monitor.monitorCollection(self.list)
        self.obj = self.klass(subject=u'New object')
        self.list.append(self.obj)

    def tearDown(self):
        self.monitor.unmonitorClass(self.klass)
        self.monitor.unmonitorCollection(self.list)


class MonitorObjectTest(MonitorBaseTest):
    def doTestAttributeChanged(self, name, value, initialValue, methodName=None):
        if methodName is None:
            methodName = name
        getattr(self.obj, 'set' + methodName[:1].upper() + methodName[1:])(initialValue)
        self.monitor.resetChanges(self.obj)
        getattr(self.obj, 'set' + methodName[:1].upper() + methodName[1:])(value)
        self.assertEqual(self.monitor.getChanges(self.obj), set([name]))

    def doTestAttributeDidNotChange(self, name, initialValue, methodName=None):
        if methodName is None:
            methodName = name
        getattr(self.obj, 'set' + methodName[:1].upper() + methodName[1:])(initialValue)
        self.monitor.resetChanges(self.obj)
        getattr(self.obj, 'set' + methodName[:1].upper() + methodName[1:])(getattr(self.obj, methodName)())
        self.assertEqual(self.monitor.getChanges(self.obj), set())

    def testSubjectChanged(self):
        self.doTestAttributeChanged('subject', 'New subject', 'Old subject')

    def testSubjectDidNotChange(self):
        self.doTestAttributeDidNotChange('subject', 'Subject')

    def testDescriptionChanged(self):
        self.doTestAttributeChanged('description', 'New description', 'Old description')

    def testDescriptionDidNotChange(self):
        self.doTestAttributeDidNotChange('description', 'Description')

    def testForegroundColorChanged(self):
        self.doTestAttributeChanged('appearance', (128, 128, 128), (64, 64, 64), 'foregroundColor')

    def testForegroundColorDidNotChange(self):
        self.doTestAttributeDidNotChange('appearance', (128, 128, 128), 'backgroundColor')

    def testBackgroundColorChanged(self):
        self.doTestAttributeChanged('appearance', (128, 128, 128), (64, 64, 64), 'backgroundColor')

    def testBackgroundColorDidNotChange(self):
        self.doTestAttributeDidNotChange('appearance', (128, 128, 128), 'backgroundColor')

    def testFontChanged(self):
        self.doTestAttributeChanged('appearance', 'dummy1', 'dummy2', 'font')

    def testFontDidNotChange(self):
        self.doTestAttributeDidNotChange('appearance', 'dummy', 'font')

    def testIconChanged(self):
        self.doTestAttributeChanged('appearance', 'foo', 'bar', 'icon')

    def testIconDidNotChange(self):
        self.doTestAttributeDidNotChange('appearance', 'foo', 'icon')

    def testSelectedIconChanged(self):
        self.doTestAttributeChanged('appearance', 'foo', 'bar', 'selectedIcon')

    def testSelectedIconDidNotChange(self):
        self.doTestAttributeDidNotChange('appearance', 'foo', 'selectedIcon')

    def testNewObject(self):
        obj = self.klass(subject=u'New')
        self.list.append(obj)
        self.assertEqual(self.monitor.getChanges(obj), None)

    def testDeletedObject(self):
        self.monitor.setChanges(self.obj.id(), set())
        self.list.remove(self.obj)
        self.failUnless(self.monitor.isRemoved(self.obj))

    def testRemoveAdd(self):
        self.monitor.resetChanges(self.obj)
        self.obj.setSubject('Foo')
        self.list.remove(self.obj)
        self.assertEqual(self.monitor.getChanges(self.obj), set(['subject', '__del__']))
        self.list.append(self.obj)
        self.assertEqual(self.monitor.getChanges(self.obj), set(['subject']))
        self.failIf(self.monitor.isRemoved(self.obj))


class MonitorCompositeObjectTest(MonitorObjectTest):
    klass = CompositeObject

    def setUp(self):
        super(MonitorCompositeObjectTest, self).setUp()

        self.child = self.klass(subject=u'Child')
        self.obj.addChild(self.child)

    def testNewChild(self):
        child = self.obj.newChild(subject='Child')
        self.assertEqual(self.monitor.getChanges(child), None)

    def testChangeChildSubject1(self):
        self.monitor.resetChanges(self.obj)
        self.child.setSubject('Child subject')
        self.assertEqual(self.monitor.getChanges(self.obj), set())

    def testChangeChildSubject2(self):
        self.monitor.resetChanges(self.child)
        self.child.setSubject('Child subject')
        self.assertEqual(self.monitor.getChanges(self.child), set(['subject']))

    def testExpansionChanged(self):
        self.monitor.resetChanges(self.obj)
        self.obj.expand()
        self.assertEqual(self.monitor.getChanges(self.obj), set(['expandedContexts']))

    def testAddChild(self):
        child = self.klass(subject='Child')
        self.list.append(child)
        self.monitor.resetChanges(child)
        self.obj.addChild(child)
        self.assertEqual(self.monitor.getChanges(child), set(['__parent__']))
